# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.
#
from unittest.mock import call

import mock
from osc_lib import exceptions

from otcextensions.osclient.css.v1 import cluster
from otcextensions.osclient.css.v1 import snapshot
from otcextensions.tests.unit.osclient.css.v1 import fakes

# from openstackclient.tests.unit import utils as tests_utils


COLUMNS = (
    'agency',
    'backup_keep_days',
    'backup_path',
    'backup_period',
    'bucket_name',
    'cmk_id',
    'is_enabled',
    'prefix',
)


class TestListSnapshots(fakes.TestCss):

    _cluster = fakes.FakeCluster.create_one()
    objects = fakes.FakeSnapshot.create_multiple(3)

    column_list_headers = (
        'ID',
        'Name',
        'Status',
        'Backup Method',
        'Bucket Name',
        'Created At',
        'Backup Keep Days',
    )

    columns = (
        'id',
        'name',
        'status',
        'backup_method',
        'bucket_name',
        'created_at',
        'backup_keep_days',
    )

    data = []

    for s in objects:
        data.append(
            (
                s.id,
                s.name,
                s.status,
                s.backup_method,
                s.bucket_name,
                s.created_at,
                s.backup_keep_days,
            )
        )

    def setUp(self):
        super(TestListSnapshots, self).setUp()

        self.cmd = snapshot.ListSnapshots(self.app, None)

        self.client.find_cluster = mock.Mock(return_value=self._cluster)
        self.client.snapshots = mock.Mock()
        self.client.api_mock = self.client.snapshots

    def test_list(self):
        arglist = [
            self._cluster.name,
        ]
        verifylist = [
            ('cluster', self._cluster.name),
        ]
        # Verify cm is triggered with default parameters
        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        # Set the response
        self.client.api_mock.side_effect = [self.objects]

        # Trigger the action
        columns, data = self.cmd.take_action(parsed_args)

        self.client.api_mock.assert_called_with(self._cluster)

        self.assertEqual(self.column_list_headers, columns)
        self.assertEqual(self.data, list(data))


class TestCreateSnapshot(fakes.TestCss):

    _cluster = fakes.FakeCluster.create_one()
    _data = fakes.FakeSnapshot.create_one()

    columns = ('id', 'name')

    data = fakes.gen_data(_data, columns)

    default_timeout = 600

    def setUp(self):
        super(TestCreateSnapshot, self).setUp()

        self.cmd = snapshot.CreateSnapshot(self.app, None)

        self.client.find_cluster = mock.Mock(return_value=self._cluster)
        self.client.create_snapshot = mock.Mock(return_value=self._data)
        self.client.wait_for_cluster = mock.Mock(return_value=True)

    def test_create(self):
        arglist = [
            self._cluster.name,
            'test-snapshot',
            '--indices',
            '1',
            '--description',
            '2',
            '--wait',
        ]
        verifylist = [
            ('cluster', self._cluster.name),
            ('name', 'test-snapshot'),
            ('indices', '1'),
            ('description', '2'),
            ('wait', True),
        ]
        # Verify cm is triggereg with default parameters
        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        # Trigger the action
        columns, data = self.cmd.take_action(parsed_args)
        attrs = {
            'name': 'test-snapshot',
            'indices': '1',
            'description': '2',
        }
        self.client.create_snapshot.assert_called_with(self._cluster, **attrs)
        self.client.wait_for_cluster.assert_called_with(
            self._cluster.id, self.default_timeout
        )
        self.assertEqual(self.columns, columns)


class TestRestoreSnapshot(fakes.TestCss):

    _cluster = fakes.FakeCluster.create_one()

    columns = (
        'action_progress',
        'actions',
        'bandwidth_size',
        'cmk_id',
        'created_at',
        'datastore',
        'elb_whitelist',
        'endpoints',
        'enterprise_project_id',
        'floating_ip',
        'id',
        'is_authority_enabled',
        'is_backup_enabled',
        'is_billed',
        'is_disk_encrypted',
        'is_https_enabled',
        'name',
        'network_id',
        'nodes',
        'router_id',
        'security_group_id',
        'status',
        'status_code',
        'tags',
        'updated_at',
    )

    data = fakes.gen_data(_cluster, columns, cluster._formatters)
    default_timeout = 1200

    def setUp(self):
        super(TestRestoreSnapshot, self).setUp()

        self.cmd = snapshot.RestoreSnapshot(self.app, None)

        self.client.find_cluster = mock.Mock(return_value=self._cluster)
        self.client.restore_snapshot = mock.Mock(return_value=None)
        self.client.wait_for_cluster = mock.Mock(return_value=True)
        self.client.get_cluster = mock.Mock(return_value=self._cluster)

    def test_restore(self):
        arglist = [
            self._cluster.name,
            'snapshot-uuid',
            '--target-cluster',
            self._cluster.name,
            '--indices',
            '1',
            '--rename-replacement',
            '2',
            '--rename-pattern',
            '3',
            '--wait',
        ]
        verifylist = [
            ('cluster', self._cluster.name),
            ('snapshotId', 'snapshot-uuid'),
            ('target_cluster', self._cluster.name),
            ('indices', '1'),
            ('rename_replacement', '2'),
            ('rename_pattern', '3'),
            ('wait', True),
        ]
        # Verify cm is triggereg with default parameters
        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        # Trigger the action
        columns, data = self.cmd.take_action(parsed_args)
        attrs = {
            'targetCluster': self._cluster.id,
            'indices': '1',
            'renameReplacement': '2',
            'renamePattern': '3',
        }
        self.client.restore_snapshot.assert_called_with(
            self._cluster, 'snapshot-uuid', **attrs
        )
        self.client.wait_for_cluster.assert_called_with(
            self._cluster.id, self.default_timeout
        )
        self.assertEqual(self.columns, columns)


class TestDeleteSnapshot(fakes.TestCss):

    _cluster = fakes.FakeCluster.create_one()
    _data = fakes.FakeSnapshot.create_multiple(2)

    def setUp(self):
        super(TestDeleteSnapshot, self).setUp()

        self.client.find_cluster = mock.Mock(return_value=self._cluster)
        self.client.delete_snapshot = mock.Mock(return_value=None)

        # Get the command object to test
        self.cmd = snapshot.DeleteSnapshot(self.app, None)

    def test_delete(self):
        arglist = [
            self._cluster.name,
            self._data[0].id,
        ]

        verifylist = [
            ('cluster', self._cluster.name),
            ('snapshot', [self._data[0].id]),
        ]

        # Verify cm is triggered with default parameters
        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        # Trigger the action
        result = self.cmd.take_action(parsed_args)
        self.client.delete_snapshot.assert_called_with(
            self._cluster, self._data[0].id, ignore_missing=False
        )
        self.assertIsNone(result)

    def test_multiple_delete(self):
        arglist = [self._cluster.name]

        snapshot_list = []
        for css_snapshot in self._data:
            snapshot_list.append(css_snapshot.id)
            arglist.append(css_snapshot.id)

        verifylist = [
            ('cluster', self._cluster.name),
            ('snapshot', snapshot_list),
        ]

        # Verify cm is triggered with default parameters
        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        # Trigger the action
        result = self.cmd.take_action(parsed_args)

        calls = []
        for css_snapshot in self._data:
            calls.append(
                call(self._cluster, css_snapshot.id, ignore_missing=False)
            )
        self.client.delete_snapshot.assert_has_calls(calls)
        self.assertIsNone(result)

    def test_multiple_delete_with_exception(self):
        arglist = [
            self._cluster.name,
            self._data[0].id,
            'unexist_css_snapshot',
        ]
        verifylist = [
            ('cluster', self._cluster.name),
            ('snapshot', [self._data[0].id, 'unexist_css_snapshot']),
        ]

        # Verify cm is triggered with default parameters
        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        delete_mock_results = [None, exceptions.CommandError]
        self.client.delete_snapshot = mock.Mock(
            side_effect=delete_mock_results
        )

        # Trigger the action
        try:
            self.cmd.take_action(parsed_args)
        except Exception as e:
            self.assertEqual('1 of 2 Snapshot(s) failed to delete.', str(e))

        self.client.delete_snapshot.assert_any_call(
            self._cluster, self._data[0].id, ignore_missing=False
        )


class TestSetSnapshotPolicy(fakes.TestCss):

    _cluster = fakes.FakeCluster.create_one()
    _data = fakes.FakeSnapshotPolicy.create_one()

    columns = COLUMNS

    data = fakes.gen_data(_data, columns)

    def setUp(self):
        super(TestSetSnapshotPolicy, self).setUp()

        self.cmd = snapshot.SetSnapshotPolicy(self.app, None)

        self.client.find_cluster = mock.Mock(return_value=self._cluster)
        self.client.set_snapshot_policy = mock.Mock(return_value=None)
        self.client.get_snapshot_policy = mock.Mock(return_value=self._data)

    def test_setpolicy(self):
        arglist = [
            self._cluster.name,
            '--name-prefix',
            '1',
            '--keep-days',
            '2',
            '--period',
            '3',
            '--frequency',
            'DAY',
            '--disable',
            '--delete-auto',
        ]
        verifylist = [
            ('cluster', self._cluster.name),
            ('name_prefix', '1'),
            ('keep_days', 2),
            ('period', '3'),
            ('frequency', 'day'),
            ('disable', True),
            ('delete_auto', True),
        ]
        # Verify cm is triggereg with default parameters
        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        # Trigger the action
        columns, data = self.cmd.take_action(parsed_args)
        attrs = {
            "prefix": "1",
            "keepday": 2,
            "period": "3",
            "frequency": "DAY",
            "enable": 'false',
            "deleteAuto": 'true',
        }
        self.client.set_snapshot_policy.assert_called_with(
            self._cluster, **attrs
        )
        self.client.get_snapshot_policy.assert_called_with(self._cluster)

        self.assertEqual(self.columns, columns)


class TestShowSnapshotPolicy(fakes.TestCss):

    _cluster = fakes.FakeCluster.create_one()
    _data = fakes.FakeSnapshotPolicy.create_one()

    columns = COLUMNS

    data = fakes.gen_data(_data, columns)

    def setUp(self):
        super(TestShowSnapshotPolicy, self).setUp()

        self.cmd = snapshot.ShowSnapshotPolicy(self.app, None)

        self.client.find_cluster = mock.Mock(return_value=self._cluster)
        self.client.get_snapshot_policy = mock.Mock(return_value=self._data)

    def test_showpolicy(self):
        arglist = [
            self._cluster.name,
        ]
        verifylist = [
            ('cluster', self._cluster.name),
        ]
        # Verify cm is triggereg with default parameters
        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        # Trigger the action
        columns, data = self.cmd.take_action(parsed_args)
        self.client.get_snapshot_policy.assert_called_with(self._cluster)

        self.assertEqual(self.columns, columns)
        self.assertEqual(self.data, data)


class TestConfigureSnapshot(fakes.TestCss):

    _cluster = fakes.FakeCluster.create_one()

    def setUp(self):
        super(TestConfigureSnapshot, self).setUp()

        self.cmd = snapshot.ConfigureSnapshot(self.app, None)
        self.client.find_cluster = mock.Mock(return_value=self._cluster)

    def test_configure(self):
        arglist = [
            self._cluster.name,
            '--backup-path',
            'css_repository/test-css',
            '--bucket',
            'test-bucket',
            '--agency',
            'test-agency',
            '--cmk-id',
            'cmk-uuid',
        ]
        verifylist = [
            ('cluster', self._cluster.name),
            ('backup_path', 'css_repository/test-css'),
            ('bucket', 'test-bucket'),
            ('agency', 'test-agency'),
            ('cmk_id', 'cmk-uuid'),
        ]
        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        attrs = {
            'backup_path': 'css_repository/test-css',
            'bucket': 'test-bucket',
            'agency': 'test-agency',
            'snapshotCmkId': 'cmk-uuid',
        }
        # Trigger the action
        result = self.cmd.take_action(parsed_args)
        self.client.set_snapshot_configuration.assert_called_with(
            self._cluster, auto_configure=False, **attrs
        )
        self.assertIsNone(result)

    def test_configure_auto(self):
        arglist = [self._cluster.name, '--auto-configure']
        verifylist = [
            ('cluster', self._cluster.name),
            ('auto_configure', True),
        ]
        # Verify cm is triggereg with default parameters
        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        # Trigger the action
        result = self.cmd.take_action(parsed_args)
        self.client.set_snapshot_configuration.assert_called_with(
            self._cluster, auto_configure=True
        )
        self.assertIsNone(result)


class TestDisableSnapshot(fakes.TestCss):

    _cluster = fakes.FakeCluster.create_one()

    def setUp(self):
        super(TestDisableSnapshot, self).setUp()

        self.client.find_cluster = mock.Mock(return_value=self._cluster)
        self.client.disable_snapshot_function = mock.Mock(return_value=None)

        # Get the command object to test
        self.cmd = snapshot.DisableSnapshot(self.app, None)

    def test_disable(self):
        arglist = [
            self._cluster.name,
        ]

        verifylist = [
            ('cluster', self._cluster.name),
        ]

        # Verify cm is triggered with default parameters
        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        # Trigger the action
        result = self.cmd.take_action(parsed_args)
        self.client.disable_snapshot_function.assert_called_with(
            self._cluster
        )
        self.assertIsNone(result)



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.
#
import mock

from otcextensions.osclient.dcaas.v2 import virtual_gateway
from otcextensions.tests.unit.osclient.dcaas.v2 import fakes

from osc_lib import exceptions

from openstackclient.tests.unit import utils as tests_utils


class TestListVirtualGateway(fakes.TestDcaas):

    objects = fakes.FakeVirtualGateway.create_multiple(3)

    column_list_headers = (
        'id',
        'name',
        'vpc id',
        'local ep group id',
        'type',
        'status'
    )
    columns = (
        'id',
        'name',
        'vpc id',
        'local ep group id',
        'type',
        'status'
    )
    data = []
    for s in objects:
        data.append((
            s.id,
            s.name,
            s.vpc_id,
            s.local_ep_group_id,
            s.type,
            s.status
        ))

    def setUp(self):
        super(TestListVirtualGateway, self).setUp()
        self.cmd = virtual_gateway.ListVirtualGateways(self.app, None)
        self.client.virtual_gateways = mock.Mock()
        self.client.api_mock = self.client.virtual_gateways

    def test_list(self):
        arglist = []
        verifylist = []

        parsed_args = self.check_parser(self.cmd, arglist, verifylist)
        self.client.api_mock.side_effect = [self.objects]
        columns, data = self.cmd.take_action(parsed_args)

        self.client.api_mock.assert_called_with()
        self.assertEqual(self.column_list_headers, columns)
        self.assertEqual(self.data, list(data))

    def test_list_args(self):
        arglist = [
            '--id', '1',
            '--name', 'test',
            '--vpc_id', 'vpc_id1',
            '--local_ep_group_id', 'lepg_id1',
            '--type', 'double ipsec',
            '--status', 'ACTIVE'
        ]
        verifylist = [
            ('id', '1'),
            ('name', 'test'),
            ('vpc_id', 'vpc_id1'),
            ('local_ep_group_id', 'lepg_id1'),
            ('type', 'double ipsec'),
            ('status', 'ACTIVE')
        ]

        parsed_args = self.check_parser(self.cmd, arglist, verifylist)
        self.client.api_mock.side_effect = [self.objects]
        columns, data = self.cmd.take_action(parsed_args)

        self.client.api_mock.assert_called_with(
            id='1',
            name='test',
            vpc_id='vpc_id1',
            local_ep_group_id='lepg_id1',
            type='double ipsec',
            status='ACTIVE'
        )


class TestCreateVirtualGateway(fakes.TestDcaas):
    _data = fakes.FakeVirtualGateway.create_one()

    columns = (
        'admin_state_up',
        'bgp_asn',
        'description',
        'device_id',
        'id',
        'ipsec_bandwidth',
        'local_ep_group_id',
        'name',
        'redundant_device_id',
        'status',
        'type',
        'vpc_id'
    )

    data = fakes.gen_data(_data, columns)

    def setUp(self):
        super(TestCreateVirtualGateway, self).setUp()
        self.cmd = virtual_gateway.CreateVirtualGateway(self.app, None)
        self.client.create_virtual_gateway = mock.Mock(return_value=self._data)

    def test_create(self):
        arglist = [
            'vpc_id1',
            'lepg_id1',
            '--name', 'test',
            '--description', 'test description',
            '--device_id', 'device_id1',
            '--redundant_device_id', 'red_dev_id1',
            '--type', 'default',
            '--ipsec_bandwidth', '50',
            '--bgp_asn', '10',
            '--admin_state_up', 'True'
        ]
        verifylist = [
            ('vpc_id', 'vpc_id1'),
            ('local_ep_group_id', 'lepg_id1'),
            ('name', 'test'),
            ('description', 'test description'),
            ('device_id', 'device_id1'),
            ('redundant_device_id', 'red_dev_id1'),
            ('type', 'default'),
            ('ipsec_bandwidth', 50),
            ('bgp_asn', 10),
            ('admin_state_up', True)
        ]

        parsed_args = self.check_parser(self.cmd, arglist, verifylist)
        columns, data = self.cmd.take_action(parsed_args)
        self.client.create_virtual_gateway.assert_called_with(
            name='test',
            vpc_id='vpc_id1',
            local_ep_group_id='lepg_id1',
            description='test description',
            device_id='device_id1',
            redundant_device_id='red_dev_id1',
            type='default',
            ipsec_bandwidth=50,
            bgp_asn=10,
            admin_state_up=True
        )
        self.assertEqual(self.columns, columns)


class TestShowVirtualGateway(fakes.TestDcaas):

    _data = fakes.FakeVirtualGateway.create_one()

    columns = (
        'admin_state_up',
        'bgp_asn',
        'description',
        'device_id',
        'id',
        'ipsec_bandwidth',
        'local_ep_group_id',
        'name',
        'redundant_device_id',
        'status',
        'type',
        'vpc_id'
    )

    data = fakes.gen_data(_data, columns)

    def setUp(self):
        super(TestShowVirtualGateway, self).setUp()
        self.cmd = virtual_gateway.ShowVirtualGateway(self.app, None)
        self.client.find_virtual_gateway = mock.Mock(return_value=self._data)

    def test_show_no_option(self):
        arglist = []
        verifylist = []
        self.assertRaises(tests_utils.ParserException,
                          self.check_parser, self.cmd, arglist, verifylist)

    def test_show(self):
        arglist = [
            self._data.id,
        ]
        verifylist = [
            ('virtual_gateway', self._data.id),
        ]

        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        columns, data = self.cmd.take_action(parsed_args)
        self.client.find_virtual_gateway.assert_called_with(self._data.id)

        self.assertEqual(self.columns, columns)
        self.assertEqual(self.data, data)

    def test_show_non_existing(self):
        arglist = [
            'unexisting_virtual_gateway'
        ]
        verifylist = [
            ('virtual_gateway', 'unexisting_virtual_gateway')
        ]

        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        find_mock_result = exceptions.CommandError('Resource Not Found')
        self.client.find_virtual_gateway = (
            mock.Mock(side_effect=find_mock_result)
        )
        try:
            self.cmd.take_action(parsed_args)
        except Exception as e:
            self.assertEqual('Resource Not Found', str(e))
        self.client.find_virtual_gateway.assert_called_with(
            'unexisting_virtual_gateway'
        )


class TestUpdateVirtualGateway(fakes.TestDcaas):

    _data = fakes.FakeVirtualGateway.create_one()

    columns = (
        'admin_state_up',
        'bgp_asn',
        'description',
        'device_id',
        'id',
        'ipsec_bandwidth',
        'local_ep_group_id',
        'name',
        'redundant_device_id',
        'status',
        'type',
        'vpc_id'
    )

    data = fakes.gen_data(_data, columns)

    def setUp(self):
        super(TestUpdateVirtualGateway, self).setUp()
        self.cmd = virtual_gateway.UpdateVirtualGateway(self.app, None)
        self.client.find_virtual_gateway = mock.Mock(return_value=self._data)
        self.client.update_virtual_gateway = mock.Mock(return_value=self._data)
        self.client.api_mock = self.client.update_virtual_gateway

    def test_update(self):
        arglist = [
            self._data.id,
            '--name', 'updated_name',
            '--description', 'updated description',
            '--local_ep_group_id', 'lepg_id2'
        ]
        verifylist = [
            ('virtual_gateway', self._data.id),
            ('name', 'updated_name'),
            ('description', 'updated description'),
            ('local_ep_group_id', 'lepg_id2')
        ]
        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        columns, data = self.cmd.take_action(parsed_args)

        self.client.api_mock.side_effect = [
            self._data
        ]

        self.client.api_mock.assert_called_with(
            self._data.id,
            name='updated_name',
            description='updated description',
            local_ep_group_id='lepg_id2'
        )
        self.assertEqual(self.columns, columns)


class TestDeleteVirtualGateway(fakes.TestDcaas):
    _data = fakes.FakeVirtualGateway.create_one()

    def setUp(self):
        super(TestDeleteVirtualGateway, self).setUp()
        self.client.delete_virtual_gateway = mock.Mock(return_value=None)
        self.cmd = virtual_gateway.DeleteVirtualGateway(self.app, None)

    def test_delete(self):
        arglist = [
            self._data.name,
        ]
        verifylist = [
            ('virtual_gateway', self._data.name)
        ]
        parsed_args = self.check_parser(self.cmd, arglist, verifylist)
        self.client.find_virtual_gateway = (
            mock.Mock(return_value=self._data)
        )
        result = self.cmd.take_action(parsed_args)
        self.client.delete_virtual_gateway.assert_called_with(self._data.id)
        self.assertIsNone(result)



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from otcextensions.tests.functional.sdk.vlb import TestVlb


class TestCertificate(TestVlb):
    def setUp(self):
        super(TestCertificate, self).setUp()
        self.create_certificate()

    def test_01_list_certificates(self):
        certs = list(self.client.certificates())
        self.assertGreaterEqual(len(certs), 0)

    def test_02_get_certificate(self):
        cert = self.client.get_certificate(TestVlb.certificate)
        self.assertEqual(TestVlb.certificate.name, cert.name)
        self.assertEqual(TestVlb.certificate.created_at, cert.created_at)
        self.assertEqual(TestVlb.certificate.expire_time, cert.expire_time)

    def test_03_find_certificate(self):
        cert = self.client.find_certificate(TestVlb.certificate.name)
        self.assertEqual(TestVlb.certificate.name, cert.name)
        self.assertEqual(TestVlb.certificate.created_at, cert.created_at)
        self.assertEqual(TestVlb.certificate.expire_time, cert.expire_time)

    def test_04_update_certificate(self):
        cert_updated = self.client.update_certificate(
            TestVlb.certificate,
            name=TestVlb.certificate.name + "_2_cp"
        )
        self.assertEqual(TestVlb.certificate.name, cert_updated.name)

        self.addCleanup(self.client.delete_certificate, TestVlb.certificate)



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from openstack.tests.unit import base

from otcextensions.sdk.dds.v3 import recycle_policy

EXAMPLE = {
    "enabled": True,
    "retention_period_in_days": 3
}


class TestRecyclePolicy(base.TestCase):
    def test_basic(self):
        sot = recycle_policy.RecyclePolicy()
        path = 'instances/recycle-policy'
        self.assertEqual(path, sot.base_path)
        self.assertTrue(sot.allow_fetch)
        self.assertTrue(sot.allow_create)

    def test_make_it(self):
        sot = recycle_policy.RecyclePolicy(**EXAMPLE)
        self.assertEqual(EXAMPLE['enabled'], sot.enabled)
        self.assertEqual(EXAMPLE['retention_period_in_days'],
                         sot.retention_period_in_days)



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.
from openstack import proxy
from openstack import resource

from otcextensions.common.utils import extract_url_parts
from otcextensions.sdk.vlb.v3 import availability_zone as _availability_zone
from otcextensions.sdk.vlb.v3 import certificate as _certificate
from otcextensions.sdk.vlb.v3 import flavor as _flavor
from otcextensions.sdk.vlb.v3 import health_monitor as _hm
from otcextensions.sdk.vlb.v3 import ip_address_group as _ip_address_group
from otcextensions.sdk.vlb.v3 import l7_policy as _l7policy
from otcextensions.sdk.vlb.v3 import l7_rule as _l7rule
from otcextensions.sdk.vlb.v3 import listener as _listener
from otcextensions.sdk.vlb.v3 import load_balancer as _lb
from otcextensions.sdk.vlb.v3 import load_balancer_status as _lb_statuses
from otcextensions.sdk.vlb.v3 import member as _member
from otcextensions.sdk.vlb.v3 import security_policy as _sp
from otcextensions.sdk.vlb.v3 import pool as _pool
from otcextensions.sdk.vlb.v3 import quota as _quota


class Proxy(proxy.Proxy):
    skip_discovery = True

    def _extract_name(self, url, service_type=None, project_id=None):
        return extract_url_parts(url, project_id)

    # ======== Load balancer ========
    def create_load_balancer(self, **attrs):
        """Create a new load balancer from attributes

        :param dict attrs: Keyword arguments which will be used to create
            a :class:`~otcextensions.sdk.vlb.v3.
            load_balancer.LoadBalancer`, comprised of the properties on the
            LoadBalancer class.

        :returns: The results of load balancer creation
        :rtype: :class:`~otcextensions.sdk.vlb.v3.load_balancer.LoadBalancer`
        """
        return self._create(_lb.LoadBalancer, **attrs)

    def get_load_balancer(self, *attrs):
        """Get a load balancer

        :param load_balancer: The value can be the name of a load balancer
             or :class:`~otcextensions.sdk.vlb.v3.load_balancer.LoadBalancer`
             instance.

        :returns: One
             :class:`~otcextensions.sdk.vlb.v3.load_balancer.LoadBalancer`
        """
        return self._get(_lb.LoadBalancer, *attrs)

    def get_load_balancer_statistics(self, name_or_id):
        """Get the load balancer statistics

        :param name_or_id: The name or ID of a load balancer

        :returns: One :class:`~otcextensions.sdk.vlb.v3.load_balancer.
                  LoadBalancerStats`
        """
        return self._get(_lb.LoadBalancerStats, lb_id=name_or_id,
                         requires_id=False)

    def load_balancers(self, **query):
        """Retrieve a generator of load balancers

        :returns: A generator of load balancer instances
        """
        return self._list(_lb.LoadBalancer, **query)

    def delete_load_balancer(self, load_balancer, ignore_missing=True):
        """Delete a load balancer

        :param load_balancer: The load_balancer can be either the id or a
            :class:`~otcextensions.sdk.vlb.v3.load_balancer.LoadBalancer`
            instance
        :param bool ignore_missing: When set to ``False``
            :class:`~openstack.exceptions.ResourceNotFound` will be raised when
            the load balancer does not exist.
            When set to ``True``, no exception will be set when attempting to
            delete a nonexistent load balancer.

        :returns: ``None``
        """
        # load_balancer = self._get_resource(_lb.LoadBalancer, load_balancer)
        return self._delete(_lb.LoadBalancer, load_balancer,
                            ignore_missing=ignore_missing)

    def find_load_balancer(self, name_or_id, ignore_missing=True):
        """Find a single load balancer

        :param name_or_id: The name or ID of a load balancer
        :param bool ignore_missing: When set to ``False``
            :class:`~openstack.exceptions.ResourceNotFound` will be raised
            when the load balancer does not exist.
            When set to ``True``, no exception will be set when attempting
            to delete a nonexistent load balancer.

        :returns: ``None``
        """
        return self._find(_lb.LoadBalancer, name_or_id,
                          ignore_missing=ignore_missing)

    def update_load_balancer(self, load_balancer, **attrs):
        """Update a load balancer

        :param load_balancer: The load_balancer can be either the name or a
            :class:`~otcextensions.sdk.vlb.v3.load_balancer.LoadBalancer`
            instance
        :param dict attrs: The attributes to update on the load balancer
                           represented by ``load_balancer``.

        :returns: The updated load_balancer
        :rtype: :class:`~otcextensions.sdk.vlb.v3.load_balancer.LoadBalancer`
        """
        return self._update(_lb.LoadBalancer, load_balancer, **attrs)

    def wait_for_load_balancer(self, name_or_id, status='ACTIVE',
                               failures=['ERROR'], interval=2, wait=300):
        lb = self._find(_lb.LoadBalancer, name_or_id, ignore_missing=False)

        return resource.wait_for_status(self, lb, status, failures, interval,
                                        wait, attribute='provisioning_status')

    def get_load_balancer_statuses(self, loadbalancer_id):
        """Get specific load balancer statuses by load balancer id.

        :param loadbalancer_id: The load balancer id
        :returns: The status of load balancer
        :rtype:
            :class:
            `~otcextensions.sdk.vlb.v3.load_balancer_status.LoadBalancerStatus`
        """
        return self._get(
            _lb_statuses.LoadBalancerStatus,
            requires_id=False,
            loadbalancer_id=loadbalancer_id
        )

    # ======== Listener ========
    def create_listener(self, **attrs):
        """Create a new listener from attributes

        :param dict attrs: Keyword arguments which will be used to create
            a :class:`~otcextensions.sdk.vlb.v3.
            listener.Listener`, comprised of the properties on the
            Listener class.

        :returns: The results of listener creation
        :rtype: :class:`~otcextensions.sdk.vlb.v3.listener.Listener`
        """
        return self._create(_listener.Listener, **attrs)

    def delete_listener(self, listener, ignore_missing=True):
        """Delete a listener

        :param listener: The value can be either the ID of a listner or a
               :class:`~otcextensions.sdk.vlb.v3.listener.Listener` instance.
        :param bool ignore_missing: When set to ``False``
                    :class:`~openstack.exceptions.ResourceNotFound` will be
                    raised when the listner does not exist.
                    When set to ``True``, no exception will be set when
                    attempting to delete a nonexistent listener.
        :returns: ``None``
        """
        self._delete(_listener.Listener, listener,
                     ignore_missing=ignore_missing)

    def update_listener(self, listener, **attrs):
        """Update a listener

        :param listener: Either the id of a listener or a
                      :class:`~otcextensions.sdk.vlb.v3.listener.Listener`
                      instance.
        :param dict attrs: The attributes to update on the listener
                           represented by ``listener``.
        :returns: The updated listener
        :rtype: :class:`~otcextensions.sdk.vlb.v3.listener.Listener`
        """
        return self._update(_listener.Listener, listener, **attrs)

    def find_listener(self, name_or_id, ignore_missing=True):
        """Find a single listener

        :param name_or_id: The name or ID of a listener.
        :param bool ignore_missing: When set to ``False``
                    :class:`~openstack.exceptions.ResourceNotFound` will be
                    raised when the resource does not exist.
                    When set to ``True``, None will be returned when
                    attempting to find a nonexistent resource.
        :returns: One :class:`~otcextensions.sdk.vlb.v3.listener.Listener`
         or None
        """
        return self._find(_listener.Listener, name_or_id,
                          ignore_missing=ignore_missing)

    def get_listener(self, listener):
        """Get a single listener

        :param listener: The value can be the ID of a listener or a
               :class:`~otcextensions.sdk.vlb.v3.listener.Listener`
               instance.
        :returns: One :class:`~otcextensions.sdk.vlb.v3.listener.Listener`
        :raises: :class:`~openstack.exceptions.ResourceNotFound`
                 when no resource can be found.
        """
        return self._get(_listener.Listener, listener)

    def listeners(self, **query):
        """Retrieve a generator of listeners

        :returns: A generator of listeners instances
        :rtype: :class:`~otcextensions.sdk.vlb.v3.listener.Listener`
        """
        return self._list(_listener.Listener, **query)

    # ======== Certificate ========
    def create_certificate(self, **attrs):
        """Create a new certificate from attributes

        :param dict attrs: Keyword arguments which will be used to create
            a :class:`~otcextensions.sdk.vlb.v3.certificate.Certificate`,
            comprised of the properties on the Certificate class.

        :returns: The results of the Certificate Creation
        :rtype: :class:`~otcextensions.sdk.vlb.v3.certificate.Certificate`
        """
        return self._create(_certificate.Certificate, **attrs)

    def certificates(self, **query):
        """Return a generator of certificates

        :param dict query: Optional query parameters to be sent to limit
            the resources being returned.
        :returns: A generator of certificates objects.
        """
        return self._list(_certificate.Certificate, **query)

    def delete_certificate(self, certificate, ignore_missing=True):
        """Delete a certificate

        :param certificate: The value can be the ID of a ELB certificate or a
            :class:`~otcextensions.sdk.vlb.v3.certificate.Certificate`
            instance.
        :param bool ignore_missing: When set to ``False``
            :class:`~openstack.exceptions.ResourceNotFound` will be raised
            when the certificate does not exist.
            When set to ``True``, no exception will be set when attempting to
            delete a nonexistent certificate.
        :returns: ``None``
        """
        return self._delete(_certificate.Certificate, certificate,
                            ignore_missing=ignore_missing)

    def get_certificate(self, certificate):
        """Get a single certificate

        :param certificate: The value can be the ID of a ELB certificate or a
            :class:`~otcextensions.sdk.vlb.v3.certificate.Certificate`
            instance.

        :returns: One :class:
            `~otcextensions.sdk.vlb.v3.certificate.Certificate`
        :raises: :class:`~openstack.exceptions.ResourceNotFound`
            when no resource can be found.
        """
        return self._get(_certificate.Certificate, certificate)

    def update_certificate(self, certificate, **attrs):
        """Update a certificate

        :param certificate: The value can be either the ID of a ELB certificate
            or a :class:`~otcextensions.sdk.vlb.v3.certificate.Certificate`
            instance.
        :param dict attrs: The attributes to update on the certificate
            represented by ``certificate``.
        :returns: The updated certificate.

        :rtype: :class:`~otcextensions.vlb.v3.certificate.Certificate`
        """
        return self._update(_certificate.Certificate, certificate, **attrs)

    def find_certificate(self, name_or_id, ignore_missing=False):
        """Find a single certificate

        :param name_or_id: The name or ID of a ELB certificate
        :param bool ignore_missing: When set to ``False``
            :class:`~openstack.exceptions.ResourceNotFound` will be raised
            when the certificate does not exist.
            When set to ``True``, no exception will be set when attempting
            to find a nonexistent certificate.
        :returns:
            One :class:`~otcextensions.sdk.vlb.v3.certificate.Certificate`
            or ``None``
        """
        return self._find(_certificate.Certificate, name_or_id,
                          ignore_missing=ignore_missing)

    # ======== Quota ========
    def get_quotas(self):
        """Retrieve a quotas

        :returns: A quota instance
        :rtype: :class:`~otcextensions.sdk.vlb.v3.quota.Quota`
        """
        return _quota.Quota.get(self)

    # ======== Availability zone ========
    def availability_zones(self, **query):
        """Retrieve a generator of availability zones

        :returns: A AvailabilityZone instance
        :rtype: :class:`AvailabilityZone`
        """

        return self._list(_availability_zone.AvailabilityZone, **query)

    # ======= Flavor =======
    def flavors(self, **query):
        """List all load balancer flavors that are
        available to a specific user in a specific region.

        :param engine_name: database engine name
        :param region: region

        :returns: A generator of flavor
        :rtype: :class:`~otcextensions.sdk.vlb.v3.flavor.Flavor`
        """

        return self._list(
            _flavor.Flavor,
            **query
        )

    def get_flavor(self, flavor):
        """Get a single flavor

        :param flavor: The value can be either the ID of an flavor or a
            :class:`~otcextensions.sdk.vlb.v3.flavor.Flavor`.

        :returns: One :class:`~otcextensions.sdk.vlb.v3.flavor.Flavor`
        """
        return self._get(_flavor.Flavor, flavor)

    def find_flavor(self, name_or_id, ignore_missing=True):
        """Find a single flavor

        :param name_or_id: The name or ID of a flavor
        :param bool ignore_missing: When set to ``False``
            :class:`~openstack.exceptions.ResourceNotFound` will be raised
            when the flavor does not exist.
            When set to ``True``, no exception will be set when attempting
            to delete a nonexistent flavor.

        :returns: ``None``
        """
        return self._find(_flavor.Flavor, name_or_id,
                          ignore_missing=ignore_missing)

    # ======= Pool =======
    def create_pool(self, **attrs):
        """Create a new pool from attributes

        :param dict attrs: Keyword arguments which will be used to create
            a :class:`~otcextensions.sdk.vlb.v3.pool.Pool`,
            comprised of the properties on the Pool class.

        :returns: The results of the Pool Creation
        :rtype: :class:`~otcextensions.sdk.vlb.v3.pool.Pool`
        """
        return self._create(_pool.Pool, **attrs)

    def get_pool(self, *attrs):
        """Get a single pool

        :param dict attrs: Keyword arguments which will be used to get a
            :class:`~otcextensions.sdk.vlb.v3.pool.Pool`
            instance.

        :returns: One :class:
            `~otcextensions.sdk.vlb.v3.pool.Pool`
        :raises: :class:`~openstack.exceptions.ResourceNotFound`
            when no resource can be found.
        """
        return self._get(_pool.Pool, *attrs)

    def pools(self, **query):
        """Return a generator of pool

        :param dict query: Optional query parameters to be sent to limit
            the resources being returned.
        :returns: A generator of pools objects.
        """
        return self._list(_pool.Pool, **query)

    def delete_pool(self, pool, ignore_missing=True):
        """Delete a pool

        :param pool: The value can be the ID of a ELB pool or a
            :class:`~otcextensions.sdk.vlb.v3.pool.Pool`
            instance.
        :param bool ignore_missing: When set to ``False``
            :class:`~openstack.exceptions.ResourceNotFound` will be raised
            when the pool does not exist.
            When set to ``True``, no exception will be set when attempting to
            delete a nonexistent pool.
        :returns: ``None``
        """
        return self._delete(_pool.Pool, pool,
                            ignore_missing=ignore_missing)

    def find_pool(self, name_or_id, ignore_missing=True):
        """Find a single pool

        :param name_or_id: The name or ID of a pool
        :param bool ignore_missing: When set to ``False``
            :class:`~openstack.exceptions.ResourceNotFound` will be raised
            when the pool does not exist.
            When set to ``True``, no exception will be set when attempting
            to delete a nonexistent pool.
        :returns: ``None``
        """
        return self._find(_pool.Pool, name_or_id,
                          ignore_missing=ignore_missing)

    def update_pool(self, pool, **attrs):
        """Update a pool

        :param pool: The value can be either the ID of a ELB pool
            or a :class:`~otcextensions.sdk.vlb.v3.pool.Pool`
            instance.
        :param dict attrs: The attributes to update on the pool
            represented by ``pool``.
        :returns: The updated pool.

        :rtype: :class:`~otcextensions.vlb.v3.pool.Pool`
        """
        return self._update(_pool.Pool, pool, **attrs)

    # ======= Member =======
    def create_member(self, pool, **attrs):
        """Create a new member from attributes

        :param pool: The pool can be either the ID of a pool or a
            :class:`~otcextensions.sdk.vlb.v3.pool.Pool` instance
            that the member will be created in.
        :param dict attrs: Keyword arguments which will be used to create
            a :class:`~otcextensions.sdk.vlb.v3.member.Member`,
            comprised of the properties on the Member class.
        :returns: The results of member creation
        :rtype: :class:`~otcextensions.sdk.vlb.v3.member.Member`
        """
        poolobj = self._get_resource(_pool.Pool, pool)
        return self._create(_member.Member, pool_id=poolobj.id,
                            **attrs)

    def delete_member(self, member, pool, ignore_missing=True):
        """Delete a member

        :param member:
            The member can be either the ID of a member or a
            :class:`~otcextensions.sdk.vlb.v3.member.Member` instance.
        :param pool: The pool can be either the ID of a pool or a
            :class:`~otcextensions.sdk.vlb.v3.pool.Pool` instance
            that the member belongs to.
        :param bool ignore_missing: When set to ``False``
            :class:`~openstack.exceptions.ResourceNotFound` will be
            raised when the member does not exist.
            When set to ``True``, no exception will be set when
            attempting to delete a nonexistent member.
        :returns: ``None``
        """
        poolobj = self._get_resource(_pool.Pool, pool)
        self._delete(_member.Member, member,
                     ignore_missing=ignore_missing, pool_id=poolobj.id)

    def find_member(self, name_or_id, pool, ignore_missing=True):
        """Find a single member

        :param str name_or_id: The name or ID of a member.
        :param pool: The pool can be either the ID of a pool or a
            :class:`~otcextensions.sdk.vlb.v3.pool.Pool` instance
            that the member belongs to.
        :param bool ignore_missing: When set to ``False``
            :class:`~openstack.exceptions.ResourceNotFound` will be
            raised when the resource does not exist.
            When set to ``True``, None will be returned when
            attempting to find a nonexistent resource.
        :returns: One :class:`~otcextensions.sdk.vlb.v3.member.Member`
            or None
        """
        poolobj = self._get_resource(_pool.Pool, pool)
        return self._find(_member.Member, name_or_id,
                          ignore_missing=ignore_missing, pool_id=poolobj.id)

    def get_member(self, member, pool):
        """Get a single member

        :param member: The member can be the ID of a member or a
            :class:`~openstack.load_balancer.v3.member.Member`
            instance.
        :param pool: The pool can be either the ID of a pool or a
            :class:`~otcextensions.sdk.vlb.v3.pool.Pool` instance
            that the member belongs to.
        :returns: One :class:`~otcextensions.sdk.vlb.v3.member.Member`
        :raises: :class:`~openstack.exceptions.ResourceNotFound`
            when no resource can be found.
        """
        poolobj = self._get_resource(_pool.Pool, pool)
        return self._get(_member.Member, member,
                         pool_id=poolobj.id)

    def members(self, pool, **query):
        """Return a generator of members

        :param pool: The pool can be either the ID of a pool or a
            :class:`~otcextensions.sdk.vlb.v3.pool.Pool` instance
            that the member belongs to.
        :param dict query: Optional query parameters to be sent to limit
            the resources being returned.
        :returns: A generator of member objects
        :rtype: :class:`~otcextensions.sdk.vlb.v3.member.Member`
        """
        poolobj = self._get_resource(_pool.Pool, pool)
        return self._list(_member.Member, pool_id=poolobj.id, **query)

    def update_member(self, member, pool, **attrs):
        """Update a member

        :param member: Either the ID of a member or a
            :class:`~otcextensions.sdk.vlb.v3.member.Member`
            instance.
        :param pool: The pool can be either the ID of a pool or a
            :class:`~otcextensions.sdk.vlb.v3.pool.Pool` instance
            that the member belongs to.
        :param dict attrs: The attributes to update on the member
            represented by ``member``.
        :returns: The updated member
        :rtype: :class:`~otcextensions.sdk.vlb.v3.member.Member`
        """
        poolobj = self._get_resource(_pool.Pool, pool)
        return self._update(_member.Member, member,
                            pool_id=poolobj.id, **attrs)

    # ======= HealthMonitor =======
    def find_health_monitor(self, name_or_id, ignore_missing=True):
        """Find a single health monitor

        :param str name_or_id: The name or ID of a health monitor.
        :param bool ignore_missing: When set to ``False``
            :class:`~openstack.exceptions.ResourceNotFound` will be
            raised when the resource does not exist.
            When set to ``True``, None will be returned when
            attempting to find a nonexistent resource.
        :returns: One
            :class:`~otcextensions.sdk.vlb.v3.healthmonitor.HealthMonitor`
            or None
        :raises: :class:`openstack.exceptions.DuplicateResource` if more
            than one resource is found for this request.
        :raises: :class:`openstack.exceptions.ResourceNotFound` if nothing
            is found and ignore_missing is ``False``.
        """
        return self._find(_hm.HealthMonitor, name_or_id,
                          ignore_missing=ignore_missing)

    def create_health_monitor(self, **attrs):
        """Create a new health monitor from attributes

        :param dict attrs: Keyword arguments which will be used to create
            a :class:`~otcextensions.sdk.vlb.v3.healthmonitor.HealthMonitor`,
            comprised of the properties on the Member class.
        :returns: The results of health monitor creation
        :rtype: :class:`~otcextensions.sdk.vlb.v3.healthmonitor.HealthMonitor`
        """
        return self._create(_hm.HealthMonitor, **attrs)

    def get_health_monitor(self, healthmonitor):
        """Get a single health monitor

        :param healthmonitor: The value can be the ID of a health monitor or a
            :class:`~openstack.load_balancer.v3.healthmonitor.HealthMonitor`
            instance.
        :returns: One
            :class:`~otcextensions.sdk.vlb.v3.healthmonitor.HealthMonitor`
        :raises: :class:`~openstack.exceptions.ResourceNotFound`
            when no resource can be found.
        """
        return self._get(_hm.HealthMonitor, healthmonitor)

    def health_monitors(self, **query):
        """Return a generator of health monitors

        :param dict query: Optional query parameters to be sent to limit
            the resources being returned.
        :returns: A generator of health monitor objects
        :rtype: :class:`~otcextensions.sdk.vlb.v3.healthmonitor.HealthMonitor`
        """
        return self._list(_hm.HealthMonitor, **query)

    def delete_health_monitor(self, healthmonitor, ignore_missing=True):
        """Delete a health monitor

        :param healthmonitor:
            The value can be either the ID of a health monitor or a
            :class:`~otcextensions.sdk.vlb.v3.health monitor.HealthMonitor`
            instance.
        :param bool ignore_missing: When set to ``False``
            :class:`~openstack.exceptions.ResourceNotFound` will be
            raised when the member does not exist.
            When set to ``True``, no exception will be set when
            attempting to delete a nonexistent health monitor.
        :returns: ``None``
        """
        return self._delete(_hm.HealthMonitor, healthmonitor,
                            ignore_missing=ignore_missing)

    def update_health_monitor(self, healthmonitor, **attrs):
        """Update a health monitor

        :param healthmonitor: The healthmonitor can be either the ID of the
            health monitor or a
            :class:`~openstack.load_balancer.v3.healthmonitor.HealthMonitor`
            instance
        :param dict attrs: The attributes to update on the health monitor
            represented by ``healthmonitor``.
        :returns: The updated health monitor
        :rtype: :class:`~otcextensions.sdk.vlb.v3.health monitor.HealthMonitor`
        """
        return self._update(_hm.HealthMonitor, healthmonitor,
                            **attrs)

    # ======= L7Policy =======
    def create_l7_policy(self, **attrs):
        """Create a new l7policy from attributes

        :param dict attrs: Keyword arguments which will be used to create
            a :class:`~otcextensions.sdk.vlb.v3.l7_policy.L7Policy`,
            comprised of the properties on the L7Policy class.
        :returns: The results of l7policy creation
        :rtype: :class:`~otcextensions.sdk.vlb.v3.l7_policy.L7Policy`
        """
        return self._create(_l7policy.L7Policy, **attrs)

    def delete_l7_policy(self, l7_policy, ignore_missing=True):
        """Delete a l7policy

        :param l7_policy:
            The value can be either the ID of a l7policy or a
            :class:`~otcextensions.sdk.vlb.v3.l7_policy.L7Policy`
            instance.
        :param bool ignore_missing: When set to ``False``
            :class:`~openstack.exceptions.ResourceNotFound` will be
            raised when the l7policy does not exist.
            When set to ``True``, no exception will be set when
            attempting to delete a nonexistent l7policy.
        :returns: ``None``
        """
        self._delete(_l7policy.L7Policy, l7_policy,
                     ignore_missing=ignore_missing)

    def find_l7_policy(self, name_or_id, ignore_missing=True):
        """Find a single l7policy

        :param str name_or_id: The name or ID of a l7policy.
        :param bool ignore_missing: When set to ``False``
            :class:`~openstack.exceptions.ResourceNotFound` will be
            raised when the resource does not exist.
            When set to ``True``, None will be returned when
            attempting to find a nonexistent resource.
        :returns: One
            :class:`~otcextensions.sdk.vlb.v3.l7_policy.L7Policy`
            or None
        :raises: :class:`openstack.exceptions.DuplicateResource` if more
            than one resource is found for this request.
        :raises: :class:`openstack.exceptions.ResourceNotFound` if nothing
            is found and ignore_missing is ``False``.
        """
        return self._find(_l7policy.L7Policy, name_or_id,
                          ignore_missing=ignore_missing)

    def get_l7_policy(self, l7_policy):
        """Get a single l7policy

        :param healthmonitor: The value can be the ID of a l7policy or a
            :class:`~openstack.load_balancer.v3.l7_policy.L7Policy`
            instance.
        :returns: One
            :class:`~otcextensions.sdk.vlb.v3.l7_policy.L7Policy`
        :raises: :class:`~openstack.exceptions.ResourceNotFound`
            when no resource can be found.
        """
        return self._get(_l7policy.L7Policy, l7_policy)

    def l7_policies(self, **query):
        """Return a generator of l7policies

        :param dict query: Optional query parameters to be sent to limit
            the resources being returned.
        :returns: A generator of l7policies objects
        :rtype: :class:`~otcextensions.sdk.vlb.v3.l7_policy.L7Policy`
        """
        return self._list(_l7policy.L7Policy, **query)

    def update_l7_policy(self, l7_policy, **attrs):
        """Update a l7policy

        :param l7_policy: The l7policy can be either the ID of the
            l7policy or a
            :class:`~openstack.load_balancer.v3.l7_policy.L7Policy`
            instance
        :param dict attrs: The attributes to update on the l7policy
            represented by ``L7policy``.
        :returns: The updated l7policy
        :rtype: :class:`~otcextensions.sdk.vlb.v3.l7_policy.L7Policy`
        """
        return self._update(_l7policy.L7Policy, l7_policy, **attrs)

    # ======= L7Rule =======
    def create_l7_rule(self, l7_policy, **attrs):
        """Create a new l7rule from attributes

        :param l7_policy: The l7_policy can be either the ID of a l7policy or
            :class:`~otcextensions.sdk.vlb.v3.l7_policy.L7Policy`
            instance that the l7rule will be created in.
        :param dict attrs: Keyword arguments which will be used to create
            a :class:`~otcextensions.sdk.vlb.v3.l7_rule.L7Rule`,
            comprised of the properties on the L7Rule class.
        :returns: The results of l7rule creation
        :rtype: :class:`~otcextensions.sdk.vlb.v3.l7_rule.L7Rule`
        """
        l7policyobj = self._get_resource(_l7policy.L7Policy, l7_policy)
        return self._create(_l7rule.L7Rule, l7policy_id=l7policyobj.id,
                            **attrs)

    def delete_l7_rule(self, l7rule, l7_policy, ignore_missing=True):
        """Delete a l7rule

        :param l7rule:
            The l7rule can be either the ID of a l7rule or a
            :class:`~otcextensions.sdk.vlb.v3.l7_rule.L7Rule` instance.
        :param l7_policy: The l7_policy can be either the ID of a l7policy or
            :class:`~otcextensions.sdk.vlb.v3.l7_policy.L7Policy`
            instance that the l7rule belongs to.
        :param bool ignore_missing: When set to ``False``
            :class:`~openstack.exceptions.ResourceNotFound` will be
            raised when the l7rule does not exist.
            When set to ``True``, no exception will be set when
            attempting to delete a nonexistent l7rule.
        :returns: ``None``
        """
        l7policyobj = self._get_resource(_l7policy.L7Policy, l7_policy)
        self._delete(_l7rule.L7Rule, l7rule, ignore_missing=ignore_missing,
                     l7policy_id=l7policyobj.id)

    def find_l7_rule(self, name_or_id, l7_policy, ignore_missing=True):
        """Find a single l7rule

        :param str name_or_id: The name or ID of a l7rule.
        :param l7_policy: The l7_policy can be either the ID of a l7policy or
            :class:`~otcextensions.sdk.vlb.v3.l7_policy.L7Policy`
            instance that the l7rule belongs to.
        :param bool ignore_missing: When set to ``False``
            :class:`~openstack.exceptions.ResourceNotFound` will be
            raised when the resource does not exist.
            When set to ``True``, None will be returned when
            attempting to find a nonexistent resource.
        :returns: One :class:`~otcextensions.sdk.vlb.v3.l7_rule.L7Rule`
            or None
        """
        l7policyobj = self._get_resource(_l7policy.L7Policy, l7_policy)
        return self._find(_l7rule.L7Rule, name_or_id,
                          ignore_missing=ignore_missing,
                          l7policy_id=l7policyobj.id)

    def get_l7_rule(self, l7rule, l7_policy):
        """Get a single l7rule

        :param l7rule: The l7rule can be the ID of a l7rule or a
            :class:`~otcextensions.sdk.vlb.v3.l7_rule.L7Rule`
            instance.
        :param l7_policy: The l7_policy can be either the ID of a l7policy or
            :class:`~otcextensions.sdk.vlb.v3.l7_policy.L7Policy`
            instance that the l7rule belongs to.
        :returns: One :class:`~otcextensions.sdk.vlb.v3.l7_rule.L7Rule`
        :raises: :class:`~openstack.exceptions.ResourceNotFound`
            when no resource can be found.
        """
        l7policyobj = self._get_resource(_l7policy.L7Policy, l7_policy)
        return self._get(_l7rule.L7Rule, l7rule,
                         l7policy_id=l7policyobj.id)

    def l7_rules(self, l7_policy, **query):
        """Return a generator of l7rules

        :param l7_policy: The l7_policy can be either the ID of a l7_policy or
            :class:`~otcextensions.sdk.vlb.v3.l7_policy.L7Policy`
            instance that the l7rule belongs to.
        :param dict query: Optional query parameters to be sent to limit
            the resources being returned. Valid parameters are:
        :returns: A generator of l7rule objects
        :rtype: :class:`~otcextensions.sdk.vlb.v3.l7_rule.L7Rule`
        """
        l7policyobj = self._get_resource(_l7policy.L7Policy, l7_policy)
        return self._list(_l7rule.L7Rule, l7policy_id=l7policyobj.id, **query)

    def update_l7_rule(self, l7rule, l7_policy, **attrs):
        """Update a l7rule

        :param l7rule: Either the ID of a l7rule or a
            :class:`~otcextensions.sdk.vlb.v3.l7_rule.L7Rule`
            instance.
        :param l7_policy: The l7_policy can be either the ID of a l7policy or
            :class:`~otcextensions.sdk.vlb.v3.l7_policy.L7Policy`
            instance that the l7rule belongs to.
        :param dict attrs: The attributes to update on the l7rule
            represented by ``l7rule``.
        :returns: The updated l7rule
        :rtype: :class:`~otcextensions.sdk.vlb.v3.l7_rule.L7Rule`
        """
        l7policyobj = self._get_resource(_l7policy.L7Policy, l7_policy)
        if 'value' in attrs:
            attrs['rule_value'] = attrs.pop('value')
        return self._update(_l7rule.L7Rule, l7rule,
                            l7policy_id=l7policyobj.id, **attrs)

    # ======== Security policy ========
    def create_security_policy(self, **attrs):
        """Create a new security policy from attributes

        :param dict attrs: Keyword arguments which will be used to create
            a :class:`~otcextensions.sdk.vlb.v3.
            security_policy.SecurityPolicy`, comprised of the properties
            on the SecurityPolicy class.

        :returns: The results of security policy creation
        :rtype: :class:`~otcextensions.sdk.vlb.v3.security_policy.
            SecurityPolicy`
        """
        return self._create(_sp.SecurityPolicy, **attrs)

    def get_security_policy(self, security_policy):
        """Get a security policy

        :param security_policy: The value can be the name of an
            security_policy or :class:`~otcextensions.sdk.vlb.v3.
            security_policy.SecurityPolicy` instance.

        :returns: One
             :class:`~otcextensions.sdk.vlb.v3.security_policy.SecurityPolicy`
        """
        return self._get(_sp.SecurityPolicy, security_policy)

    def security_policies(self, **query):
        """Retrieve a generator of security policies

        :returns: A generator of security policies
        """
        return self._list(_sp.SecurityPolicy, **query)

    def update_security_policy(self, security_policy, **attrs):
        """Update a security policy

        :param security_policy: The security_policy can be either the id or
            a :class:`~otcextensions.sdk.vlb.v3.security_policy.
            SecurityPolicy` instance
        :param dict attrs: The attributes to update on the security policy
                           represented by ``security_policy``.

        :returns: The updated security_policy
        :rtype: :class:`~otcextensions.sdk.vlb.v3.security_policy.
                SecurityPolicy`
        """
        return self._update(_sp.SecurityPolicy, security_policy, **attrs)

    def find_security_policy(self, name_or_id, ignore_missing=True):
        """Find a single security policy

        :param name_or_id: The name or ID of an security policy
        :param bool ignore_missing: When set to ``False``
            :class:`~openstack.exceptions.ResourceNotFound` will be raised
            when the security policy does not exist.
            When set to ``True``, no exception will be set when attempting
            to delete a nonexistent security policy.

        :returns: ``None``
        """
        return self._find(_sp.SecurityPolicy, name_or_id,
                          ignore_missing=ignore_missing)

    def delete_security_policy(self, security_policy, ignore_missing=True):
        """Delete a security policy

        :param security_policy: The security_policy can be either the name or a
            :class:`~otcextensions.sdk.vlb.v3.security_policy.SecurityPolicy`
            instance
        :param bool ignore_missing: When set to ``False``
            :class:`~openstack.exceptions.ResourceNotFound` will be raised when
            the security policy does not exist.
            When set to ``True``, no exception will be set when attempting to
            delete a nonexistent security policy.

        :returns: ``None``
        """
        return self._delete(_sp.SecurityPolicy, security_policy,
                            ignore_missing=ignore_missing)

    # ======== Ip address group ========
    def create_ip_address_group(self, **attrs):
        """Create a new ip address group from attributes

        :param dict attrs: Keyword arguments which will be used to create
            a :class:`~otcextensions.sdk.vlb.v3.
            ip_address_group.IpAddressGroup`, comprised of the properties
            on the IpAddressGroup class.

        :returns: The results of ip address group creation
        :rtype: :class:`~otcextensions.sdk.vlb.v3.ip_address_group.
            IpAddressGroup`
        """
        return self._create(_ip_address_group.IpAddressGroup, **attrs)

    def get_ip_address_group(self, ip_address_group):
        """Get an ip address group

        :param ip_address_group: The value can be the name of an
            ip_address_group or :class:`~otcextensions.sdk.vlb.v3.
            ip_address_group.IpAddressGroup` instance.

        :returns: One
             :class:`~otcextensions.sdk.vlb.v3.ip_address_group.IpAddressGroup`
        """
        return self._get(_ip_address_group.IpAddressGroup, ip_address_group)

    def ip_address_groups(self, **query):
        """Retrieve a generator of an ip address group

        :returns: A generator of ip address group instances
        """
        return self._list(_ip_address_group.IpAddressGroup, **query)

    def delete_ip_address_group(self, ip_address_group, ignore_missing=True):
        """Delete an ip address group

        :param ip_address_group: The ip address group can be either the name or
            a :class:`~otcextensions.sdk.vlb.v3.ip_address_group.
            IpAddressGroup` instance
        :param bool ignore_missing: When set to ``False``
            :class:`~openstack.exceptions.ResourceNotFound` will be raised when
            the ip address group does not exist.
            When set to ``True``, no exception will be set when attempting to
            delete a nonexistent ip address group.

        :returns: ``None``
        """
        ip_address_group = self._get_resource(_ip_address_group.IpAddressGroup,
                                              ip_address_group)
        return self._delete(_ip_address_group.IpAddressGroup, ip_address_group,
                            ignore_missing=ignore_missing)

    def find_ip_address_group(self, name_or_id, ignore_missing=True):
        """Find a single ip address group

        :param name_or_id: The name or ID of an ip address group
        :param bool ignore_missing: When set to ``False``
            :class:`~openstack.exceptions.ResourceNotFound` will be raised
            when the ip address group does not exist.
            When set to ``True``, no exception will be set when attempting
            to delete a nonexistent ip address group.

        :returns: ``None``
        """
        return self._find(_ip_address_group.IpAddressGroup, name_or_id,
                          ignore_missing=ignore_missing)

    def update_ip_address_group(self, ip_address_group, **attrs):
        """Update a ip address group

        :param ip_address_group: The ip_address_group can be either the name or
            a :class:`~otcextensions.sdk.vlb.v3.ip_address_group.
            IpAddressGroup` instance
        :param dict attrs: The attributes to update on the ip address group
                           represented by ``ip_address_group``.

        :returns: The updated ip_address_group
        :rtype: :class:`~otcextensions.sdk.vlb.v3.ip_address_group.
            IpAddressGroup`
        """
        return self._update(_ip_address_group.IpAddressGroup, ip_address_group,
                            **attrs)

    def update_ip_addresses_in_ip_address_group(self, ip_address_group,
                                                ip_list):
        """Update ip addresses list in an existing ip address group

        :param ip_address_group: The value can be the ID of an ip address group
            or a :class:`~otcextensions.sdk.vlb.v3.ip_address_group.
            IpAddressGroup` instance.
        :param list ip_list: The list contains the IP addresses to be
            updated in the form
            {"ip": "192.168.0.3",  "description": " your description"}
            where "description" is optional
        :returns: The results of ip address group updation
        :rtype: :class:`~otcextensions.sdk.vlb.v3.ip_address_group.
            IpAddressGroup`
        """
        ip_address_group = self._get_resource(_ip_address_group.IpAddressGroup,
                                              ip_address_group)

        return ip_address_group.update_ip_addresses(
            self, ip_list=ip_list
        )

    def delete_ip_addresses_in_ip_address_group(self, ip_address_group,
                                                ip_list):
        """Delete ip addresses list from an existing ip address group

        :param ip_address_group: The value can be the ID of a ip address group
            or a :class:`~otcextensions.sdk.vlb.v3.ip_address_group.
            IpAddressGroup` instance.
        :param list ip_list: The list contains the IP addresses to be deleted
            in the form {"ip": "192.168.0.3"}
        :returns: The results of ip address group deletion
        :rtype: :class:`~otcextensions.sdk.vlb.v3.ip_address_group.
            IpAddressGroup`
        """
        ip_address_group = self._get_resource(_ip_address_group.IpAddressGroup,
                                              ip_address_group)

        return ip_address_group.delete_ip_addresses(
            self, ip_list=ip_list
        )



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.
from keystoneauth1 import adapter

import mock

from openstack.tests.unit import base

from otcextensions.sdk.cce.v3 import job as _job


EXAMPLE = {
    "kind": "Job",
    "apiVersion": "v3",
    "metadata": {
        "uid": "397eb2b9-261a-11e9-bcf7-0255ac110e1e",
        "creationTimestamp": "2019-02-01 12:09:28.712099 +0000 UTC",
        "updateTimestamp": "2019-02-01 12:10:20.649724 +0000 UTC"
    },
    "spec": {
    },
    "status": {
        "phase": "Success"
    }
}


class TestJob(base.TestCase):

    def setUp(self):
        super(TestJob, self).setUp()
        self.sess = mock.Mock(spec=adapter.Adapter)
        self.sess.get = mock.Mock()
        self.sess.default_microversion = '1'
        self.sess._get_connection = mock.Mock(return_value=self.cloud)

        self.sot = _job.Job()

    def test_basic(self):
        sot = _job.Job()
        self.assertIsNone(sot.resource_key)
        self.assertIsNone(sot.resources_key)
        self.assertEqual('/jobs',
                         sot.base_path)
        self.assertFalse(sot.allow_list)
        self.assertFalse(sot.allow_create)
        self.assertTrue(sot.allow_fetch)
        self.assertFalse(sot.allow_commit)
        self.assertFalse(sot.allow_delete)

    def test_make_it(self):
        obj = EXAMPLE
        sot = _job.Job.existing(**obj)
        self.assertEqual(obj['metadata']['uid'], sot.id)
        self.assertEqual(obj['kind'], sot.kind)

    def test_get_status(self):
        data = EXAMPLE
        job = _job.Job(**EXAMPLE)

        self.assertEqual(data['status']['phase'], getattr(job,
                                                          'status.status'))



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from otcextensions.tests.functional.sdk.apig import TestApiG


class TestGwFeatures(TestApiG):
    def setUp(self):
        super(TestGwFeatures, self).setUp()
        self.create_gateway()
        self.gateway_id = TestGwFeatures.gateway.id
        # self.gateway_id = "be76ca6de5fe4aa7af503c03b3b44dea"
        self.addCleanup(self.delete_gateway())

    def test_get_api_quantities(self):
        api_q = self.client.get_api_quantities(
            gateway=self.gateway_id)
        self.assertIsNotNone(api_q)

    def test_get_api_group_quantities(self):
        api_group_q = self.client.get_api_group_quantities(
            gateway=self.gateway_id)
        self.assertIsNotNone(api_group_q)

    def test_get_app_quantities(self):
        api_group_q = self.client.get_app_quantities(
            gateway=self.gateway_id)
        self.assertIsNotNone(api_group_q)



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.
import uuid

from otcextensions.sdk.volume_backup.v2 import backup_policy as _backup_policy
from otcextensions.tests.functional.sdk.volume_backup import TestVbs


def create_backup_policy(client, name):
    data = {
        "name": name,
        "scheduled_policy": {
            "remain_first_backup_of_curMonth": 'Y',
            "rentention_num": 10,
            "frequency": 1,
            "start_time": "12:00",
            "status": "ON"
        }
    }
    return client.create_backup_policy(**data)


class TestBackupPolicy(TestVbs):
    BACKUP_POLICY_NAME = "SDK-" + uuid.uuid4().hex
    policy = None
    volume = None

    def setUp(self):
        super(TestBackupPolicy, self).setUp()
        self.policy = create_backup_policy(self.client,
                                           self.BACKUP_POLICY_NAME)

    def tearDown(self):
        #: delete backup policy
        if self.policy and self.volume:
            self.conn.volume_backup.unlink_resources_of_policy(
                self.policy, [self.volume.id])
        if self.policy:
            self.conn.volume_backup.delete_backup_policy(self.policy)

    def test_list_policies(self):
        policies = list(self.client.backup_policies())
        self.assertGreaterEqual(len(policies), 0)

    def test_find_policy(self):
        policy = self.client.find_backup_policy(self.policy)
        assert isinstance(policy, _backup_policy.BackupPolicy)

        # retest passing only name
        policy = self.client.find_backup_policy(self.policy.name)
        assert isinstance(policy, _backup_policy.BackupPolicy)

    def test_enable_policy(self):
        policy = self.client.enable_policy(self.policy)
        assert isinstance(policy, _backup_policy.BackupPolicy)
        self.assertEqual(policy.scheduled_policy.status, 'ON')

    def test_disable_policy(self):
        policy = self.client.disable_policy(self.policy)
        assert isinstance(policy, _backup_policy.BackupPolicy)
        self.assertEqual(policy.scheduled_policy.status, 'OFF')

    def test_execute_policy(self):
        policy = self.client.enable_policy(self.policy)
        self.client.execute_policy(policy)



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.
from openstack.tests.unit import base

from otcextensions.sdk.dns.v2 import recordset


FAKE_ID = "68d5745e-6af2-40e4-945d-fe449be00148"
EXAMPLE = {
    "id": FAKE_ID,
    "name": "example.com.",
    "type": "SOA",
    "ttl": 300,
    "records": [
        "ns1.hotrot.de. xx.example.com. (1 7200 900 1209600 300)"
    ],
    "status": "ACTIVE",
    "links": {
        "self": "some_link"
    },
    "zone_id": "2c9eb155587194ec01587224c9f90149",
    "zone_name": "example.com.",
    "create_at": "2016-11-17T11:56:03.439",
    "update_at": "2016-11-17T12:56:03.827",
    "default": True,
    "project_id": "e55c6f3dc4e34c9f86353b664ae0e70c"

}

DATA = {
    "links": {
        "self": "https://example.com/v2/zones/2/recordsets"
    },
    "recordsets": [
        {
            "id": "1",
            "name": "prod.otc.1."
        },
        {
            "id": "a3",
            "name": "prod.otc.2."
        },
        {
            "id": "a7",
            "name": "prod.otc.3."
        }
    ],
    "metadata": {
        "total_count": 7
    }
}


class TestRecordSet(base.TestCase):

    def test_basic(self):
        sot = recordset.Recordset()

        self.assertEqual('/zones/%(zone_id)s/recordsets', sot.base_path)

        self.assertTrue(sot.allow_list)
        self.assertTrue(sot.allow_fetch)
        self.assertTrue(sot.allow_create)
        self.assertTrue(sot.allow_delete)
        self.assertTrue(sot.allow_commit)

    def test_make_it(self):

        sot = recordset.Recordset(**EXAMPLE)
        self.assertEqual(EXAMPLE['id'], sot.id)
        self.assertEqual(EXAMPLE['name'], sot.name)
        self.assertEqual(EXAMPLE['type'], sot.type)
        self.assertEqual(EXAMPLE['ttl'], sot.ttl)
        self.assertEqual(EXAMPLE['status'], sot.status)
        self.assertEqual(EXAMPLE['zone_id'], sot.zone_id)
        self.assertEqual(EXAMPLE['create_at'], sot.created_at)
        self.assertEqual(EXAMPLE['update_at'], sot.updated_at)

    def test_get_next_link(self):
        uri = '/zones/2/recordsets'
        marker = 'a7'

        result = recordset.Recordset._get_next_link(
            uri=uri,
            response=None,
            data=DATA,
            marker=marker,
            limit=None,
            total_yielded=3
        )
        self.assertEqual(uri, result[0])
        self.assertEqual({'marker': marker}, result[1])

    def test_get_next_link2(self):
        uri = '/zones/2/recordsets'
        marker = 'a7'

        result = recordset.Recordset._get_next_link(
            uri=uri,
            response=None,
            data=DATA,
            marker=marker,
            limit=None,
            total_yielded=7
        )
        self.assertEqual(None, result[0])
        self.assertEqual({}, result[1])

    def test_get_next_link3(self):
        uri = '/zones/2/recordsets'
        marker = 'a7'
        limit = 3

        result = recordset.Recordset._get_next_link(
            uri=uri,
            response=None,
            data=DATA,
            marker=marker,
            limit=3,
            total_yielded=3
        )
        self.assertEqual(uri, result[0])
        self.assertEqual({'marker': marker, 'limit': limit}, result[1])



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.
import uuid

import openstack

from otcextensions.tests.functional import base

_logger = openstack._log.setup_logging('openstack')


class TestContainer(base.BaseFunctionalTest):
    uuid_v4 = uuid.uuid4().hex[:8]
    bucket_name = 'obs-test-' + uuid_v4
    object_name = f'obs{uuid_v4}.object'
    folder_name = f'folder{uuid_v4}/'
    data = str(uuid.uuid4())
    container = None
    object = None

    def setUp(self):
        super(TestContainer, self).setUp()
        self.client = self.conn.obs
        self.container = self.client.create_container(
            name=self.bucket_name,
            storage_acl='public-read-write',
            storage_class='STANDARD'
        )
        self.object = self.client.create_object(
            container=self.container,
            name=self.object_name,
            data=self.data
        )
        self.folder = self.client.create_object(
            container=self.container,
            name=self.folder_name
        )

    def test_01_get_object(self):
        object = self.client.get_object(
            self.object_name,
            container=self.container
        )
        self.assertIsNotNone(object)

    def test_02_get_object_metadata(self):
        object = self.client.get_object_metadata(
            self.object_name,
            container=self.container
        )

        self.assertIsNotNone(object)
        self.assertIsNotNone(object.etag)
        self.client.delete_object(self.object)
        self.client.delete_object(self.folder)
        self.client.delete_container(self.container)



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.
from openstack import exceptions
from openstack import resource
from openstack import utils


class FlavorSpec(resource.Resource):
    #: Specifies node quantity.
    #: *Type:int*
    num = resource.Body('num', type=int)
    #: Specifies the disk size.
    #: valid value: ``ULTRAHIGH``,
    #: *Type:int*
    size = resource.Body('size', type=int)
    #: Specifies the resource specification code.
    #: *Type:string*
    spec_code = resource.Body('spec_code')
    #: Specifies the disk type.
    #: Valid values:
    #: mongos: The value ranges from 2 to 16.
    #: shard: The value ranges from 2 to 16.
    #: config: The value is 1.
    #: replica: The value is 1.
    #: *Type:string*
    storage = resource.Body('storage')
    #: Specifies the node type.
    #: * mongos
    #: * shard
    #: * config
    #: * replica
    #: *Type:string*
    type = resource.Body('type')


class BackupStrategySpec(resource.Resource):
    #: Specifies the number of days to retain the generated backup files.
    #: valid value range is from 0 to 732.
    #: *Type:string*
    keep_days = resource.Body('keep_days')
    #: Specifies the backup time window.
    #: valid value must be in the "hh:mm-HH:MM" format.
    #: Example value: 08:15-09:15
    #: *Type:string*
    start_time = resource.Body('start_time')


class VolumeSpec(resource.Resource):
    #: Disk size.
    #: *Type:string*
    size = resource.Body('size')
    #: Disk usage.
    #: *Type:string*
    used = resource.Body('used')


class NodeSpec(resource.Resource):
    #: The AZ.
    #: *Type:string*
    availability_zone = resource.Body('availability_zone')
    #: Node ID.
    #: *Type:string*
    id = resource.Body('id')
    #: Node name.
    #: *Type:string*
    name = resource.Body('name')
    #: Private IP address of a node.
    #: *Type:string*
    private_ip = resource.Body('private_ip')
    #: The EIP that has been bound.
    #: *Type:string*
    public_ip = resource.Body('public_ip')
    #: Node role.
    #: *Type:string*
    role = resource.Body('role')
    #: Resource specifications code.
    #: *Type:string*
    spec_code = resource.Body('spec_code')
    #: Node status.
    #: *Type:string*
    status = resource.Body('status')


class GroupSpec(resource.Resource):
    #: Group ID.
    #: *Type:string*
    id = resource.Body('id')
    #: Group name.
    #: *Type:string*
    name = resource.Body('name')
    #: Node information.
    #: *Type:list*
    nodes = resource.Body('nodes', type=list, list_type=NodeSpec)
    #: Group status.
    #: *Type:string*
    status = resource.Body('status')
    #: Node type.
    #: *Type:string*
    type = resource.Body('type')
    #: Volume information.
    #: *Type:dict*
    volume = resource.Body('volume', type=VolumeSpec)


class DatastoreSpec(resource.Resource):
    #: Specifies the storage engine.
    #: *Type:string*
    storage_engine = resource.Body('storage_engine')
    #: Specifies the database type.
    #: The value is ``DDS-Community``.
    #: *Type:string*
    type = resource.Body('type')
    #: Specifies the database version.
    #: valid value is 3.2 or 3.4.
    #: *Type:string*
    version = resource.Body('version')


class Instance(resource.Resource):
    base_path = '/instances'

    resources_key = 'instances'
    resource_key = 'instance'

    # capabilities
    allow_create = True
    allow_delete = True
    allow_commit = True
    allow_list = True
    allow_fetch = True

    _query_mapping = resource.QueryParameters(
        'id', 'name', 'mode', 'datastore_type',
        'vpc_id', 'subnet_id', 'limit', 'offset')

    #: Operations that is executed on the DB instance.
    #: *Type:string*
    actions = resource.Body('actions', type=list)
    #: Specifies the AZ ID.
    #: *Type:string*
    availability_zone = resource.Body('availability_zone')
    #: Specifies the advanced backup policy.
    #: *Type:dict*
    backup_strategy = resource.Body('backup_strategy', type=BackupStrategySpec)
    #: Time when a DB instance is created.
    #: *Type:string*
    created = resource.Body('created')
    #: Specifies the database information.
    #: *Type:dict*
    datastore = resource.Body('datastore', type=DatastoreSpec)
    #: Specifies the database type.
    #: *Type:string*
    datastore_type = resource.Body('datastore_type')
    #: Specifies the key ID used for disk encryption.
    #: *Type:string*
    disk_encryption_id = resource.Body('disk_encryption_id')
    #: Specifies the storage engine.
    #: *Type:string*
    engine = resource.Body('engine')
    #: Specifies the instance specifications.
    #: *Type:list*
    flavor = resource.Body('flavor', type=list, list_type=FlavorSpec)
    #: Group information
    #: *Type:dict*
    groups = resource.Body('groups', type=list, list_type=GroupSpec)
    #: DB instance ID.
    #: *Type:string*
    id = resource.Body('id')
    #: Async job id
    #: *Type:uuid*
    job_id = resource.Body('job_id')
    #: Maintenance time window.
    #: *Type:string*
    maintenance_window = resource.Body('maintenance_window')
    #: Specifies the instance type.
    #: * Sharding indicates the cluster instance.
    #: * ReplicaSet indicate the replica set instance.
    #: *Type:string*
    mode = resource.Body('mode')
    #: DB instance name.
    #: *Type:string*
    name = resource.Body('name')
    #: Specifies the database password.
    #: *Type:string*
    password = resource.Body('password')
    #: Billing mode.
    #: *Type:string*
    pay_mode = resource.Body('pay_mode')
    #: Database port number.
    #: *Type:int*
    port = resource.Body('port')
    #: Specifies the region ID.
    #: *Type:string*
    region = resource.Body('region')
    #: Specifies the ID of the security group
    #: where a specified DB instance belongs to.
    #: *Type:string*
    security_group_id = resource.Body('security_group_id')
    #: Instance status.
    #: *Type:string*
    status = resource.Body('status')
    #: Specifies whether to enable SSL.
    #: *Type:string*
    ssl = resource.Body('ssl')
    #: Data store information.
    #: *Type:string*
    subnet_id = resource.Body('subnet_id')
    #: Time zone.
    #: *Type:string*
    time_zone = resource.Body('time_zone')
    #: Specifies the VPC ID.
    #: *Type:string*
    vpc_id = resource.Body('vpc_id')
    #: Time when a DB instance is updated.
    #: *Type:string*
    updated = resource.Body('updated')

    def fetch(self, session, requires_id=True,
              base_path=None, error_message=None, **params):
        """Get a remote resource based on this instance.
        :param session: The session to use for making this request.
        :type session: :class:`~keystoneauth1.adapter.Adapter`
        :param boolean requires_id: A boolean indicating whether resource ID \
        should be part of the requested URI.
        :param str base_path: Base part of the URI for fetching resources, if \
        different from :data:`~openstack.resource.Resource.base_path`.
        :param str error_message: An Error message to be returned if \
        requested object does not exist.
        :param dict params: Additional parameters that can be consumed.

        :return: This :class:`Resource` instance.
        :raises: :exc:`~openstack.exceptions.MethodNotSupported` if \
        :data:`Resource.allow_fetch` is not set to ``True``.
        :raises: :exc:`~openstack.exceptions.ResourceNotFound` if \
        the resource was not found.

        """
        data = self.list(session, paginated=False, id=self.id)
        result = self._get_one_match(self.id, data)
        if not result:
            raise exceptions.ResourceNotFound(
                "No Instance found for %s" % (self.id))

        self._body.attributes.update(result._body.attributes)
        self._body.clean()
        return self

    def create(self, session, prepend_key=False, base_path=None):
        return super(Instance, self).create(
            session,
            prepend_key=prepend_key,
            base_path=base_path)

    def restart(self, session):
        '''Restart Instance'''
        body = {
            "target_type": self.datastore_type,
            "target_id": self.id
        }
        response = self._action(session, body, 'restart')
        exceptions.raise_from_response(response)
        self._translate_response(response)
        return self

    def enlarge(self, session, size, group_id):
        '''Enlarge Instance Storage Space'''
        body = {
            "volume":
                {
                    "size": size
                }
        }
        if group_id is not None:
            body["volume"]["group_id"] = group_id
        response = self._action(session, body, 'enlarge-volume')
        exceptions.raise_from_response(response)
        self._translate_response(response)
        return self

    def add_nodes(self, session, node_type, spec_code, num, volume=None):
        '''Add Nodes to Instance'''
        body = {
            "type": node_type,
            "spec_code": spec_code,
            "num": num
        }
        if volume is not None:
            body['volume'] = volume
        response = self._action(session, body, 'enlarge')
        exceptions.raise_from_response(response)
        self._translate_response(response)
        return self

    def resize(self, session, target_id, spec_code, target_type=None):
        body = {
            "resize": {
                "target_id": target_id,
                "target_spec_code": spec_code
            }
        }
        if target_type is not None:
            body['resize']['target_type'] = target_type
        response = self._action(session, body, 'resize')
        exceptions.raise_from_response(response)
        self._translate_response(response)
        return self

    def switchover(self, session):
        response = self._action(session, {}, 'switchover')
        exceptions.raise_from_response(response)
        self._translate_response(response)
        return self

    def switch_ssl(self, session, enable):
        body = {'ssl_option': "1" if enable else "0"}
        response = self._action(session, body, 'switch-ssl')
        exceptions.raise_from_response(response)
        self._translate_response(response)
        return self

    def modify_name(self, session, name):
        body = {
            "new_instance_name": name
        }
        response = self._action(session, body, 'modify-name', 'PUT')
        exceptions.raise_from_response(response)

    def change_port(self, session, port):
        body = {
            "port": port
        }
        response = self._action(session, body, 'modify-port')
        exceptions.raise_from_response(response)
        self._translate_response(response)
        return self

    def change_security_group(self, session, security_group_id):
        body = {
            "security_group_id": security_group_id
        }
        response = self._action(session, body, 'modify-security-group')
        exceptions.raise_from_response(response)
        self._translate_response(response)
        return self

    def change_private_ip(self, session, node_id, new_ip):
        body = {
            "node_id": node_id,
            "new_ip": new_ip
        }
        response = self._action(session, body, 'modify-internal-ip')
        exceptions.raise_from_response(response)
        self._translate_response(response)
        return self

    def create_ip(self, session, dds_type, password):
        body = {
            "type": dds_type,
            "password": password
        }
        response = self._action(session, body, 'create-ip')
        exceptions.raise_from_response(response)
        self._translate_response(response)
        return self

    def configure_client_network(self, session, network_ranges):
        body = {
            "client_network_ranges": network_ranges
        }
        response = self._action(session, body, 'client-network')
        exceptions.raise_from_response(response)
        self._translate_response(response)
        return self

    def set_recycle_bin_policy(self, session, *attrs):
        body = {
            "recycle_policy": attrs
        }
        response = self._action(session, body, 'recycle-policy')
        exceptions.raise_from_response(response)
        self._translate_response(response)
        return self

    def _action(self, session, body, action_type, api_type='POST'):
        """Preform actions given the message body.
        """
        url = utils.urljoin(self.base_path, self.id, action_type)
        if api_type == 'POST':
            return session.post(
                url,
                json=body)
        if api_type == 'PUT':
            return session.put(url, json=body)



#   Licensed under the Apache License, Version 2.0 (the "License"); you may
#   not use this file except in compliance with the License. You may obtain
#   a copy of the License at
#
#        http://www.apache.org/licenses/LICENSE-2.0
#
#   Unless required by applicable law or agreed to in writing, software
#   distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#   WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#   License for the specific language governing permissions and limitations
#   under the License.
import mock

# from otcextensions.common import sdk_utils
from otcextensions.osclient.anti_ddos.v1 import status
from otcextensions.tests.unit.osclient.anti_ddos.v1 import fakes


class TestListFloatingIPEvents(fakes.TestAntiDDoS):

    objects = fakes.FakeFloatingIPEvent.create_multiple(3)

    columns = (
        'start_time', 'end_time', 'status', 'trigger_bps',
        'trigger_pps', 'trigger_http_pps'
    )

    data = []

    for s in objects:
        data.append((
            s.start_time,
            s.end_time,
            s.status,
            s.trigger_bps,
            s.trigger_pps,
            s.trigger_http_pps
        ))

    def setUp(self):
        super(TestListFloatingIPEvents, self).setUp()

        self.cmd = status.ListFloatingIPEvents(self.app, None)

        self.client.floating_ip_events = mock.Mock()
        self.client.api_mock = self.client.floating_ip_events

    def test_list(self):
        arglist = [
            'ip_id'
        ]

        verifylist = [
            ('floating_ip', 'ip_id')
        ]

        # Verify cm is triggereg with default parameters
        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        # Set the response
        self.client.api_mock.side_effect = [
            self.objects
        ]

        # Trigger the action
        columns, data = self.cmd.take_action(parsed_args)

        self.client.api_mock.assert_called_once_with(
            floating_ip_id='ip_id'
        )

        self.assertEqual(self.columns, columns)
        self.assertEqual(self.data, list(data))


class TestListFloatingIPDayStat(fakes.TestAntiDDoS):

    objects = fakes.FakeFloatingIPStatDay.create_multiple(3)

    columns = (
        'period_start', 'bps_in', 'bps_attack', 'total_bps',
        'pps_in', 'pps_attack', 'total_pps'
    )

    data = []

    for s in objects:
        data.append((
            s.period_start,
            s.bps_in,
            s.bps_attack,
            s.total_bps,
            s.pps_in,
            s.pps_attack,
            s.total_pps
        ))

    def setUp(self):
        super(TestListFloatingIPDayStat, self).setUp()

        self.cmd = status.ListFloatingIPStatDay(self.app, None)

        self.client.floating_ip_stat_day = mock.Mock()
        self.client.api_mock = self.client.floating_ip_stat_day

    def test_list(self):
        arglist = [
            'ip_id'
        ]

        verifylist = [
            ('floating_ip', 'ip_id')
        ]

        # Verify cm is triggereg with default parameters
        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        # Set the response
        self.client.api_mock.side_effect = [
            self.objects
        ]

        # Trigger the action
        columns, data = self.cmd.take_action(parsed_args)

        self.client.api_mock.assert_called_once_with(
            floating_ip_id='ip_id'
        )

        self.assertEqual(self.columns, columns)
        self.assertEqual(self.data, list(data))


class TestFloatingIPWeekStat(fakes.TestAntiDDoS):

    _data = fakes.FakeFloatingIPStatWeek.create_one()

    columns = (
        'period_start_date', 'ddos_intercept_times', 'ddos_blackhole_times',
        'max_attack_bps', 'max_attack_conns')

    data = []
    for wd in _data.weekdata:
        data.append((
            wd.period_start_date,
            wd.ddos_intercept_times,
            wd.ddos_blackhole_times,
            wd.max_attack_bps,
            wd.max_attack_conns
        ))

    def setUp(self):
        super(TestFloatingIPWeekStat, self).setUp()

        self.cmd = status.ListFloatingIPStatWeek(self.app, None)

        self.client.floating_ip_stat_week = mock.Mock()
        self.client.api_mock = self.client.floating_ip_stat_week

    def test_default(self):
        arglist = [
        ]
        verifylist = [
        ]
        # Verify cm is triggereg with default parameters
        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        # Set the response
        self.client.api_mock.side_effect = [
            self._data
        ]

        # Trigger the action
        columns, data = self.cmd.take_action(parsed_args)

        self.client.api_mock.assert_called_with(
        )

        self.assertEqual(self.columns, columns)
        self.assertEqual(self.data, list(data))



#   Licensed under the Apache License, Version 2.0 (the "License"); you may
#   not use this file except in compliance with the License. You may obtain
#   a copy of the License at
#
#        http://www.apache.org/licenses/LICENSE-2.0
#
#   Unless required by applicable law or agreed to in writing, software
#   distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#   WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#   License for the specific language governing permissions and limitations
#   under the License.
#
'''CTS Tracker v1 action implementations'''
import logging

from osc_lib import utils
from osc_lib.command import command

from otcextensions.common import sdk_utils

from otcextensions.i18n import _

LOG = logging.getLogger(__name__)

OPERATION_VALUES = ['create', 'delete', 'login']


def _get_columns(item):
    column_map = {
    }
    return sdk_utils.get_osc_show_columns_for_sdk_resource(item, column_map)


class ShowTracker(command.ShowOne):
    _description = _('Show details of a CTS tracker')

    def get_parser(self, prog_name):
        parser = super(ShowTracker, self).get_parser(prog_name)
        parser.add_argument(
            'tracker',
            metavar='<tracker>',
            default='system',
            help=_('Tracker name (currently only `system`)')
        )
        return parser

    def take_action(self, parsed_args):
        client = self.app.client_manager.cts

        data = client.get_tracker(
            tracker=parsed_args.tracker,
        )

        display_columns, columns = _get_columns(data)
        data = utils.get_item_properties(data, columns)

        return (display_columns, data)


class DeleteTracker(command.Command):
    _description = _('Delete CTS Tracker')

    def get_parser(self, prog_name):
        parser = super(DeleteTracker, self).get_parser(prog_name)
        parser.add_argument(
            'tracker',
            metavar='<tracker>',
            nargs='+',
            help=_('Name or ID of the tracker to delete.')
        )
        return parser

    def take_action(self, parsed_args):

        if parsed_args.tracker:
            client = self.app.client_manager.cts
            for tracker in parsed_args.tracker:
                client.delete_tracker(tracker=tracker, ignore_missing=False)


class CreateTracker(command.ShowOne):
    _description = _('Create a single CTS tracker')

    def get_parser(self, prog_name):
        parser = super(CreateTracker, self).get_parser(prog_name)
        parser.add_argument(
            '--bucket_name',
            metavar='<bucket>',
            required=True,
            help=_('Specifies the OBS bucket name. The value is a string of '
                   '0 to 64 characters and can contain uppercase and '
                   'lowercase letters (a to z and A to Z), digits (0 to '
                   '9), hyphens (-), underscores (_), and periods (.). '
                   'In addition, it must start and end with a letter.')
        )
        parser.add_argument(
            '--file_prefix_name',
            metavar='<file_prefix_name>',
            help=_('Specifies the prefix of a log that needs to be stored '
                   'in an OBS  bucket. The value is a string of 0 to 64 '
                   'characters and can contain uppercase and lowercase '
                   'letters (a to z and A to Z), digits (0 to 9), '
                   'hyphens (-), underscores (_), and periods (.)')
        )
        parser.add_argument(
            '--enable_smn',
            action='store_true',
            help=_('Specifies whether SMN is supported. When the value is '
                   '`false`, `topic_id` and `operations` can be left empty.')
        )
        parser.add_argument(
            '--topic_id',
            metavar='<topic>',
            help=_('topic_id is obtained from SMN and in the format of '
                   'urn:smn: ([A-Za-z0-9-]){1,32}:'
                   '([A-Za-z0-9]){32}:'
                   '([A-Za-z0-9]|[_\\-]){1,256}.')
        )
        parser.add_argument(
            '--operation',
            metavar='{' + ','.join(OPERATION_VALUES) + '}',
            type=lambda s: s.lower(),
            choices=OPERATION_VALUES,
            action='append',
            help=_('Specifies trigger conditions for sending a notification '
                   'when Typical is selected. You can select `Delete`, '
                   '`Create`, or `Login` or all of them via repetition. '
                   'Specifies trigger conditions for sending a notification '
                   'when `--send_all_key` is selected. All conditions '
                   'including `Delete`, `Create`, `Change`, and `OpenStack '
                   'API Event` are selected by default. '
                   'Modification is not allowed.')
        )
        parser.add_argument(
            '--send_all_key',
            action='store_true',
            help=_('You can select Typical or All for Trigger Condition.\n'
                   'When the value is `false`, `operations` cannot be left '
                   'empty. When the value is `true`, operations is not '
                   'supported.')
        )
        parser.add_argument(
            '--notify_user',
            metavar='<user>',
            action='append',
            help=_('In Typical scenario, you can specify the users using the '
                   'login function. When these users log in, notifications '
                   'will be sent.')
        )
        return parser

    def take_action(self, parsed_args):

        client = self.app.client_manager.cts

        attrs = {}

        if parsed_args.bucket_name:
            attrs['bucket_name'] = parsed_args.bucket_name
        if parsed_args.file_prefix_name:
            attrs['file_prefix_name'] = parsed_args.file_prefix_name
        smn = {'enable': False}
        if parsed_args.enable_smn:
            smn['enable'] = True
            if parsed_args.topic_id:
                smn['topic_id'] = parsed_args.topic_id
            if parsed_args.send_all_key:
                smn['topic_id'] = parsed_args.topic_id
            if parsed_args.operation:
                smn['operations'] = parsed_args.operation
            if parsed_args.notify_user:
                smn['notify_users'] = parsed_args.notify_user
        attrs['smn'] = smn

        obj = client.create_tracker(**attrs)

        display_columns, columns = _get_columns(obj)
        data = utils.get_item_properties(obj, columns)

        return (display_columns, data)


class SetTracker(command.ShowOne):
    _description = _('Update single CTS tracker properties')

    def get_parser(self, prog_name):
        parser = super(SetTracker, self).get_parser(prog_name)
        parser.add_argument(
            'tracker',
            metavar='<tracker>',
            help=_('Specifies the name of the tracker. Currently only '
                   '`system` is supported.')
        )
        parser.add_argument(
            '--bucket_name',
            metavar='<bucket>',
            required=True,
            help=_('Specifies the OBS bucket name. The value is a string of '
                   '0 to 64 characters and can contain uppercase and '
                   'lowercase letters (a to z and A to Z), digits (0 to '
                   '9), hyphens (-), underscores (_), and periods (.). '
                   'In addition, it must start and end with a letter.')
        )
        parser.add_argument(
            '--file_prefix_name',
            metavar='<file_prefix_name>',
            help=_('Specifies the prefix of a log that needs to be stored '
                   'in an OBS  bucket. The value is a string of 0 to 64 '
                   'characters and can contain uppercase and lowercase '
                   'letters (a to z and A to Z), digits (0 to 9), '
                   'hyphens (-), underscores (_), and periods (.)')
        )
        parser.add_argument(
            '--enable_smn',
            action='store_true',
            help=_('Specifies whether SMN is supported. When the value is '
                   '`false`, `topic_id` and `operations` can be left empty.')
        )
        parser.add_argument(
            '--topic_id',
            metavar='<topic>',
            help=_('topic_id is obtained from SMN and in the format of '
                   'urn:smn: ([A-Za-z0-9-]){1,32}:'
                   '([A-Za-z0-9]){32}:'
                   '([A-Za-z0-9]|[_\\-]){1,256}.')
        )
        parser.add_argument(
            '--operation',
            metavar='{' + ','.join(OPERATION_VALUES) + '}',
            type=lambda s: s.lower(),
            choices=OPERATION_VALUES,
            action='append',
            help=_('Specifies trigger conditions for sending a notification '
                   'when Typical is selected. You can select `Delete`, '
                   '`Create`, or `Login` or all of them via repetition. '
                   'Specifies trigger conditions for sending a notification '
                   'when `--send_all_key` is selected. All conditions '
                   'including `Delete`, `Create`, `Change`, and `OpenStack '
                   'API Event` are selected by default. '
                   'Modification is not allowed.')
        )
        parser.add_argument(
            '--send_all_key',
            action='store_true',
            help=_('You can select Typical or All for Trigger Condition.\n'
                   'When the value is `false`, `operations` cannot be left '
                   'empty. When the value is `true`, operations is not '
                   'supported.')
        )
        parser.add_argument(
            '--notify_user',
            metavar='<user>',
            action='append',
            help=_('In Typical scenario, you can specify the users using the '
                   'login function. When these users log in, notifications '
                   'will be sent.')
        )
        group = parser.add_mutually_exclusive_group()
        group.add_argument(
            '--enable',
            action='store_true',
            help=_('Enable tracing into the bucket')
        )
        group.add_argument(
            '--disable',
            action='store_true',
            help=_('Disable tracing into the bucket')
        )
        return parser

    def take_action(self, parsed_args):

        client = self.app.client_manager.cts

        attrs = {}

        if parsed_args.bucket_name:
            attrs['bucket_name'] = parsed_args.bucket_name
        if parsed_args.file_prefix_name:
            attrs['file_prefix_name'] = parsed_args.file_prefix_name
        if parsed_args.enable:
            attrs['status'] = 'enabled'
        elif parsed_args.disable:
            attrs['status'] = 'disabled'
        smn = {'enable': False}
        if parsed_args.enable_smn:
            smn['enable'] = True
            if parsed_args.topic_id:
                smn['topic_id'] = parsed_args.topic_id
            if parsed_args.send_all_key:
                smn['topic_id'] = parsed_args.topic_id
            if parsed_args.operation:
                smn['operations'] = parsed_args.operation
            if parsed_args.notify_user:
                smn['notify_users'] = parsed_args.notify_user
        attrs['smn'] = smn

        obj = client.update_tracker(tracker=parsed_args.tracker, **attrs)

        display_columns, columns = _get_columns(obj)
        data = utils.get_item_properties(obj, columns)

        return (display_columns, data)



#   Licensed under the Apache License, Version 2.0 (the "License"); you may
#   not use this file except in compliance with the License. You may obtain
#   a copy of the License at
#
#        http://www.apache.org/licenses/LICENSE-2.0
#
#   Unless required by applicable law or agreed to in writing, software
#   distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#   WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#   License for the specific language governing permissions and limitations
#   under the License.
#
'''DNS Recordset v2 action implementations'''
import logging

from osc_lib import utils
from osc_lib.command import command

from otcextensions.i18n import _
from otcextensions.common import sdk_utils

LOG = logging.getLogger(__name__)

RS_TYPES = ['A', 'AAAA', 'MX', 'TXT', 'CNAME', 'NS', 'SRV', 'PTR', 'CAA']


_formatters = {
}


def _get_columns(item):
    column_map = {
    }
    hidden = ['location', 'links']
    return sdk_utils.get_osc_show_columns_for_sdk_resource(item, column_map,
                                                           hidden)


class ListRS(command.Lister):
    _description = _('List recordsets.')
    columns = (
        'id', 'name', 'type', 'status', 'description', 'records'
    )

    def get_parser(self, prog_name):
        parser = super(ListRS, self).get_parser(prog_name)
        parser.add_argument(
            'zone',
            metavar='<zone>',
            help=_('UUID or name of the zone. Recordsets of all zones '
                   'will be returned if not given.')
        )
        parser.add_argument(
            '--zone-type',
            help=_('DNS Zone type (private/public)')
        )
        return parser

    def take_action(self, parsed_args):
        client = self.app.client_manager.dns

        zone = None
        attrs = {}
        if parsed_args.zone_type:
            attrs['zone_type'] = parsed_args.zone_type

        if parsed_args.zone:

            zone = client.find_zone(parsed_args.zone, ignore_missing=False,
                                    **attrs)

        data = client.recordsets(zone=zone)

        table = (self.columns,
                 (utils.get_item_properties(
                     s, self.columns, formatters=_formatters
                 ) for s in data))
        return table


class ShowRS(command.ShowOne):
    _description = _('Show the recordset details')

    def get_parser(self, prog_name):
        parser = super(ShowRS, self).get_parser(prog_name)

        parser.add_argument(
            'zone',
            metavar='<zone>',
            help=_('UUID or name of the zone.')
        )
        parser.add_argument(
            '--zone-type',
            help=_('DNS Zone type (private/public)')
        )

        parser.add_argument(
            'recordset',
            metavar='<rs>',
            help=_('UUID or name of the recordset.')
        )

        return parser

    def take_action(self, parsed_args):

        client = self.app.client_manager.dns

        zone = client.find_zone(
            parsed_args.zone,
            ignore_missing=False,
            zone_type=parsed_args.zone_type
        )

        obj = client.find_recordset(zone=zone,
                                    name_or_id=parsed_args.recordset)

        display_columns, columns = _get_columns(obj)
        data = utils.get_item_properties(obj, columns)

        return (display_columns, data)


class DeleteRS(command.Command):
    _description = _('Delete Recordset')

    def get_parser(self, prog_name):
        parser = super(DeleteRS, self).get_parser(prog_name)

        parser.add_argument(
            'zone',
            metavar='<zone>',
            help=_('UUID or name of the zone.')
        )
        parser.add_argument(
            '--zone-type',
            help=_('DNS Zone type (private/public)')
        )

        parser.add_argument(
            'recordset',
            metavar='<rs>',
            nargs='+',
            help=_('UUID or Name of the recordset.')
        )

        return parser

    def take_action(self, parsed_args):
        if parsed_args.zone:
            client = self.app.client_manager.dns
            zone = client.find_zone(parsed_args.zone, ignore_missing=False,
                                    zone_type=parsed_args.zone_type)
            for rs in parsed_args.recordset:
                rs = client.find_recordset(zone=zone, name_or_id=rs,
                                           ignore_missing=False)
                client.delete_recordset(
                    recordset=rs, zone=zone, ignore_missing=False)


class CreateRS(command.ShowOne):
    _description = _('Create recordset')

    def get_parser(self, prog_name):
        parser = super(CreateRS, self).get_parser(prog_name)

        parser.add_argument(
            'zone',
            metavar='<zone>',
            help=_('UUID or name of the zone.')
        )
        parser.add_argument(
            '--zone-type',
            help=_('DNS Zone type (private/public)')
        )
        parser.add_argument(
            '--name',
            metavar='<name>',
            required=True,
            help=_('DNS Name for the zone.')
        )
        parser.add_argument(
            '--description',
            metavar='<description>',
            help=_('Description for this zone.')
        )
        parser.add_argument(
            '--type',
            metavar='{' + ','.join(RS_TYPES) + '}',
            type=lambda s: s.upper(),
            choices=RS_TYPES,
            help=_('Recordset type.')
        )
        parser.add_argument(
            '--ttl',
            metavar='<300-2147483647>',
            type=int,
            # NOTE: py2 does not support such big int, skip unless py3-only
            # choices=range(300, 2147483647),
            help=_('TTL (Time to Live) for the zone.')
        )
        parser.add_argument(
            '--record',
            metavar='<value>',
            action='append',
            help=_('Record set value, which varies depending on the record '
                   'set type. For example, the value of an `AAAA` record set '
                   'is the IPv6 address list mapped to the domain name. ')
        )

        return parser

    def take_action(self, parsed_args):

        client = self.app.client_manager.dns

        attrs = {'records': []}

        zone = client.find_zone(parsed_args.zone, ignore_missing=False,
                                zone_type=parsed_args.zone_type)

        if parsed_args.name:
            attrs['name'] = parsed_args.name
        if parsed_args.description:
            attrs['description'] = parsed_args.description
        if parsed_args.type:
            attrs['type'] = parsed_args.type
        if parsed_args.ttl:
            attrs['ttl'] = parsed_args.ttl
        for rec in parsed_args.record:
            attrs['records'].append(rec)

        obj = client.create_recordset(
            zone=zone,
            **attrs
        )

        display_columns, columns = _get_columns(obj)
        data = utils.get_item_properties(obj, columns)

        return (display_columns, data)


class SetRS(command.ShowOne):
    _description = _('Update a Recordset')

    def get_parser(self, prog_name):
        parser = super(SetRS, self).get_parser(prog_name)

        parser.add_argument(
            'zone',
            metavar='<zone>',
            help=_('UUID or name of the zone.')
        )
        parser.add_argument(
            '--zone-type',
            help=_('DNS Zone type (private/public)')
        )
        parser.add_argument(
            'recordset',
            metavar='<rs>',
            help=_('UUID or name of the recordset.')
        )
        parser.add_argument(
            '--description',
            metavar='<description>',
            help=_('Description for this zone.')
        )
        parser.add_argument(
            '--ttl',
            metavar='<300-2147483647>',
            type=int,
            # NOTE: py2 does not support such big int, skip unless py3-only
            # choices=range(300, 2147483647),
            help=_('TTL (Time to Live) for the zone.')
        )
        parser.add_argument(
            '--record',
            metavar='<value>',
            action='append',
            help=_('Record set value, which varies depending on the record '
                   'set type. For example, the value of an `AAAA` record set '
                   'is the IPv6 address list mapped to the domain name. ')
        )

        return parser

    def take_action(self, parsed_args):

        client = self.app.client_manager.dns

        attrs = {'records': []}

        if parsed_args.description:
            attrs['description'] = parsed_args.description
        if parsed_args.ttl:
            attrs['ttl'] = parsed_args.ttl

        zone = client.find_zone(parsed_args.zone, ignore_missing=False,
                                zone_type=parsed_args.zone_type)

        if parsed_args.ttl:
            attrs['zone_id'] = zone.id

        for rec in parsed_args.record:
            attrs['records'].append(rec)

        recordset = client.find_recordset(zone=zone,
                                          name_or_id=parsed_args.recordset)

        obj = client.update_recordset(
            recordset=recordset,
            **attrs
        )

        display_columns, columns = _get_columns(obj)
        data = utils.get_item_properties(obj, columns)

        return (display_columns, data)



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.
#
"""VPC Endpoint Service v1 action implementations"""
import logging

from osc_lib import utils
from osc_lib.command import command

from otcextensions.common import sdk_utils
from otcextensions.i18n import _

LOG = logging.getLogger(__name__)


def _get_columns(item):
    column_map = {}
    hidden = [
        'location',
    ]
    return sdk_utils.get_osc_show_columns_for_sdk_resource(
        item, column_map, hidden
    )


class ListWhitelist(command.Lister):

    _description = _('List whitelist records of a VPC endpoint service.')
    columns = ('Id', 'Permission', 'Created At')

    def get_parser(self, prog_name):
        parser = super(ListWhitelist, self).get_parser(prog_name)

        parser.add_argument(
            'service',
            metavar='<service>',
            help=_('ID or name of the VPC Endpoint Service.'),
        )
        parser.add_argument(
            '--sort-key',
            metavar='{created_at}',
            type=lambda s: s.lower(),
            choices=['created_at'],
            help=_('Sorting field of the whitelist records.'),
        )
        parser.add_argument(
            '--sort-dir',
            metavar='{asc, desc}',
            type=lambda s: s.lower(),
            choices=['asc', 'desc'],
            help=_('Sorting order of the whitelist record list.'),
        )
        parser.add_argument(
            '--limit',
            metavar='<limit>',
            type=int,
            help=_('Limit number of whitelist records to fetch.'),
        )
        parser.add_argument(
            '--offset',
            metavar='<offset>',
            type=int,
            help=_('Whitelist records after this Offset will be queried.'),
        )
        return parser

    def take_action(self, parsed_args):
        client = self.app.client_manager.vpcep
        args_list = [
            'limit',
            'offset',
            'sort_key',
            'sort_dir',
        ]
        attrs = {}
        for arg in args_list:
            val = getattr(parsed_args, arg)
            if val:
                attrs[arg] = val
        endpoint_service = client.find_service(parsed_args.service)
        data = client.service_whitelist(endpoint_service, **attrs)

        return (
            self.columns,
            (utils.get_item_properties(s, self.columns) for s in data),
        )


class ManageWhitelist(command.Lister):
    _description = _('Manage whitelist records of a VPC endpoint service.')

    columns = ('Permission',)

    def get_parser(self, prog_name):
        parser = super(ManageWhitelist, self).get_parser(prog_name)

        parser.add_argument(
            'service',
            metavar='<service>',
            help=_('ID or name of the VPC Endpoint Service.'),
        )
        parser.add_argument(
            'domain',
            metavar='<domain>',
            nargs='+',
            help=_(
                'Domain ID(s) to add to whitelist record of the '
                'Vpc endpoint service.'
            ),
        )
        manage_request_group = parser.add_mutually_exclusive_group(
            required=True
        )
        manage_request_group.add_argument(
            '--add',
            action='store_true',
            help=(
                'Add a domian to the whitelist record of the '
                'Vpc endpoint service.'
            ),
        )
        manage_request_group.add_argument(
            '--remove',
            action='store_true',
            help=(
                'Remove a domian from the whitelist record of the '
                'Vpc endpoint service.'
            ),
        )
        return parser

    def take_action(self, parsed_args):
        client = self.app.client_manager.vpcep
        set_args = ('add', 'remove')
        request_status = [
            request for request in set_args if getattr(parsed_args, request)
        ]

        endpoint_service = client.find_service(parsed_args.service)
        data = client.manage_service_whitelist(
            endpoint_service,
            domains=parsed_args.domain,
            action=request_status[0],
        )
        return (
            self.columns,
            (utils.get_item_properties(s, self.columns) for s in data),
        )



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from keystoneauth1 import adapter
import mock
from openstack.tests.unit import base

from otcextensions.sdk.sdrs.v1 import quota as _quota


EXAMPLE = {
    'resources': [
        {
            'type': 'server_groups',
            'used': 2,
            'quota': 10,
            'min': 0,
            'max': -1
        },
        {
            'type': 'replications',
            'used': 1,
            'quota': 100,
            'min': 0,
            'max': -1
        }
    ]
}


class TestQuota(base.TestCase):

    def setUp(self):
        super(TestQuota, self).setUp()
        self.sess = mock.Mock(spec=adapter.Adapter)
        self.sess.get = mock.Mock()
        self.sess.default_microversion = '1'
        self.sess._get_connection = mock.Mock(return_value=self.cloud)
        self.sot = _quota.Quota()

    def test_basic(self):
        sot = _quota.Quota()
        self.assertEqual('quotas', sot.resources_key)
        self.assertEqual('/sdrs/quotas',
                         sot.base_path)
        self.assertTrue(sot.allow_list)
        self.assertFalse(sot.allow_create)
        self.assertFalse(sot.allow_fetch)
        self.assertFalse(sot.allow_commit)
        self.assertFalse(sot.allow_delete)

    def test_make_it(self):
        test_quota = _quota.Quota(**EXAMPLE)
        self.assertEqual(EXAMPLE['resources'][0]['type'],
                         test_quota.resources[0].type)
        self.assertEqual(EXAMPLE['resources'][0]['used'],
                         test_quota.resources[0].used)
        self.assertEqual(EXAMPLE['resources'][0]['quota'],
                         test_quota.resources[0].quota)
        self.assertEqual(EXAMPLE['resources'][0]['min'],
                         test_quota.resources[0].min)
        self.assertEqual(EXAMPLE['resources'][0]['max'],
                         test_quota.resources[0].max)
        self.assertEqual(EXAMPLE['resources'][1]['type'],
                         test_quota.resources[1].type)
        self.assertEqual(EXAMPLE['resources'][1]['used'],
                         test_quota.resources[1].used)
        self.assertEqual(EXAMPLE['resources'][1]['quota'],
                         test_quota.resources[1].quota)
        self.assertEqual(EXAMPLE['resources'][1]['min'],
                         test_quota.resources[1].min)
        self.assertEqual(EXAMPLE['resources'][1]['max'],
                         test_quota.resources[1].max)



#   Licensed under the Apache License, Version 2.0 (the "License"); you may
#   not use this file except in compliance with the License. You may obtain
#   a copy of the License at
#
#        http://www.apache.org/licenses/LICENSE-2.0
#
#   Unless required by applicable law or agreed to in writing, software
#   distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#   WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#   License for the specific language governing permissions and limitations
#   under the License.
#
import mock

from otcextensions.osclient.cce.v1 import cluster
from otcextensions.tests.unit.osclient.cce.v1 import fakes


class TestCluster(fakes.TestCCE):

    def setUp(self):
        super(TestCluster, self).setUp()

    def test_flatten(self):
        _obj = fakes.FakeCluster.create_one()

        flat_data = cluster._flatten_cluster(_obj)

        data = (
            flat_data['id'],
            flat_data['name'],
            flat_data['cpu'],
            flat_data['memory'],
            flat_data['endpoint'],
            flat_data['availability_zone'],
            flat_data['vpc'],
            flat_data['status'],
            flat_data['nodes']
        )

        cmp_data = (
            _obj.metadata.id,
            _obj.metadata.name,
            _obj.spec.cpu,
            _obj.spec.memory,
            _obj.spec.endpoint,
            _obj.spec.availability_zone,
            _obj.spec.vpc,
            _obj.status['status'],
            len(_obj.spec.host_list.spec.host_list)
        )

        self.assertEqual(data, cmp_data)

    def test_flatten_no_nodes(self):
        _obj = fakes.FakeCluster.create_one(0)

        # remove hosts to ensure count is properly calculated
        # _obj._update(spec={'host_list': {'spec': {'host_list': []}}})

        flat_data = cluster._flatten_cluster(_obj)

        data = (
            flat_data['id'],
            flat_data['name'],
            flat_data['cpu'],
            flat_data['memory'],
            flat_data['endpoint'],
            flat_data['availability_zone'],
            flat_data['vpc'],
            flat_data['status'],
            flat_data['nodes']
        )

        cmp_data = (
            _obj.metadata.id,
            _obj.metadata.name,
            _obj.spec.cpu,
            _obj.spec.memory,
            _obj.spec.endpoint,
            _obj.spec.availability_zone,
            _obj.spec.vpc,
            _obj.status['status'],
            0
        )

        self.assertEqual(data, cmp_data)


class TestListCluster(fakes.TestCCE):

    _objs = fakes.FakeCluster.create_multiple(3)

    columns = ('ID', 'name', 'cpu', 'memory', 'endpoint')

    data = []

    for s in _objs:
        flat_data = cluster._flatten_cluster(s)
        data.append((
            flat_data['id'],
            flat_data['name'],
            flat_data['cpu'],
            flat_data['memory'],
            flat_data['endpoint'],
        ))

    def setUp(self):
        super(TestListCluster, self).setUp()

        self.cmd = cluster.ListCCECluster(self.app, None)

        self.client.clusters = mock.Mock()

    def test_list_default(self):
        arglist = []

        verifylist = []

        # Verify cm is triggereg with default parameters
        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        # Set the response
        self.client.clusters.side_effect = [
            self._objs
        ]

        # Trigger the action
        columns, data = self.cmd.take_action(parsed_args)

        self.client.clusters.assert_called_once_with()

        self.assertEqual(self.columns, columns)
        self.assertEqual(self.data, list(data))


class TestShowCluster(fakes.TestCCE):

    _obj = fakes.FakeCluster.create_one()

    columns = ('ID', 'name', 'status', 'cpu', 'memory', 'endpoint',
               'availability_zone',
               'vpc', 'nodes')
    flat_data = cluster._flatten_cluster(_obj)
    data = (
        flat_data['id'],
        flat_data['name'],
        flat_data['status'],
        flat_data['cpu'],
        flat_data['memory'],
        flat_data['endpoint'],
        flat_data['availability_zone'],
        flat_data['vpc'],
        flat_data['nodes']
    )

    def setUp(self):
        super(TestShowCluster, self).setUp()

        self.cmd = cluster.ShowCCECluster(self.app, None)

        self.client.find_cluster = mock.Mock()

    def test_get(self):
        arglist = [
            'cluster_uuid'
        ]

        verifylist = [
            ('cluster', 'cluster_uuid')
        ]

        # Verify cm is triggereg with default parameters
        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        # Set the response
        self.client.find_cluster.side_effect = [
            self._obj
        ]

        # Trigger the action
        columns, data = self.cmd.take_action(parsed_args)

        self.client.find_cluster.assert_called_once_with('cluster_uuid')

        self.assertEqual(self.columns, columns)
        self.assertEqual(self.data, data)


class TestCreateCluster(fakes.TestCCE):

    _obj = fakes.FakeCluster.create_one()

    columns = ('ID', 'name', 'status', 'cpu', 'memory', 'endpoint',
               'availability_zone',
               'vpc', 'nodes')

    flat_data = cluster._flatten_cluster(_obj)
    data = (
        flat_data['id'],
        flat_data['name'],
        flat_data['status'],
        flat_data['cpu'],
        flat_data['memory'],
        flat_data['endpoint'],
        flat_data['availability_zone'],
        flat_data['vpc'],
        flat_data['nodes']
    )

    def setUp(self):
        super(TestCreateCluster, self).setUp()

        self.cmd = cluster.CreateCCECluster(self.app, None)

        self.client.create_cluster = mock.Mock()

    def test_create(self):
        arglist = [
            'cluster_name',
            'vpc_id',
            'subnet_id',
            '--description', 'descr',
            '--region', 'regio',
            '--security_group', 'sg',
            '--type', 'HA'
        ]

        verifylist = [
            ('name', 'cluster_name'),
            ('vpc', 'vpc_id'),
            ('subnet', 'subnet_id'),
            ('description', 'descr'),
            ('region', 'regio'),
            ('security_group', 'sg'),
            ('type', 'HA')
        ]

        # Verify cm is triggereg with default parameters
        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        # Set the response
        self.client.create_cluster.side_effect = [
            self._obj
        ]

        # Trigger the action
        columns, data = self.cmd.take_action(parsed_args)

        self.client.create_cluster.assert_called_once_with(
            name='cluster_name',
            spec={
                'vpc': 'vpc_id',
                'subnet': 'subnet_id',
                'region': 'regio',
                'description': 'descr',
                'security_group_id': 'sg',
                'cluster_type': 'HA'
            }
        )

        self.assertEqual(self.columns, columns)
        self.assertEqual(self.data, data)


class TestDeleteCluster(fakes.TestCCE):

    def setUp(self):
        super(TestDeleteCluster, self).setUp()

        self.cmd = cluster.DeleteCCECluster(self.app, None)

        self.client.delete_cluster = mock.Mock()

    def test_delete(self):
        arglist = [
            'cluster_uuid'
        ]

        verifylist = [
            ('cluster', 'cluster_uuid')
        ]

        # Verify cm is triggereg with default parameters
        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        # Set the response
        self.client.delete_cluster.side_effect = [{}]

        # Trigger the action
        self.cmd.take_action(parsed_args)

        self.client.delete_cluster.assert_called_once_with('cluster_uuid')



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from openstack.tests.unit import base

from otcextensions.sdk.swr.v2 import domain

EXAMPLE = {
    'namespace': 'space',
    'repository': 'repo',
    'access_domain': 'OTC00000000001000000447',
    'permit': 'read',
    'deadline': 'forever',
    'description': 'desc'
}


class TestDomain(base.TestCase):

    def test_basic(self):
        sot = domain.Domain()
        path = 'manage/namespaces/%(namespace)s/' \
               'repositories/%(repository)s/access-domains'
        self.assertEqual(path, sot.base_path)
        self.assertTrue(sot.allow_list)
        self.assertTrue(sot.allow_create)
        self.assertTrue(sot.allow_commit)
        self.assertTrue(sot.allow_fetch)
        self.assertTrue(sot.allow_delete)

    def test_make_it(self):
        sot = domain.Domain(**EXAMPLE)
        self.assertEqual(EXAMPLE['access_domain'], sot.access_domain)
        self.assertEqual(EXAMPLE['namespace'], sot.namespace)
        self.assertEqual(EXAMPLE['deadline'], sot.deadline)
        self.assertEqual(EXAMPLE['description'], sot.description)



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.
from openstack import resource


class ErrorInfoSpec(resource.Resource):
    #: Properties
    #: Error code
    code = resource.Body('code')
    #: Error message
    message = resource.Body('message')


class ExtendInfoBackup(resource.Resource):
    #: Properties
    #: Error code returned if application-consistent backup fails
    app_consistency_error_code = resource.Body('app_consistency_error_code')
    #: Error message returned if application-consistent backup fails
    app_consistency_error_mess = resource.Body('app_consistency_error_message')
    #: Application-consistent backup status
    app_consistency_status = resource.Body('app_consistency_status')
    #: Backup ID
    backup_id = resource.Body('backup_id')
    #: Backup name
    backup_name = resource.Body('backup_name')
    #: Whether incremental backup is used
    incremental = resource.Body('incremental')


class ExtendInfoCommon(resource.Resource):
    #: Properties
    #: Progress of the query task
    progress = resource.Body('progress')
    #: Request ID
    request_id = resource.Body('request_id')
    #: Backup task ID
    task_id = resource.Body('task_id')


class ExtendInfoDelete(resource.Resource):
    #: Properties
    #: Backup ID
    backup_id = resource.Body('backup_id')
    #: Backup name
    backup_name = resource.Body('backup_name')


class ExtendInfoSync(resource.Resource):
    #: Properties
    #: Number of synchronized backups
    sync_backup_num = resource.Body('sync_backup_num')
    #: Number of deleted backups
    delete_backup_num = resource.Body('delete_backup_num')
    #: Number of backups that fail to be synchronized
    err_sync_backup_num = resource.Body('err_sync_backup_num')


class ExtendInfoRestore(resource.Resource):
    #: Properties
    #: Backup ID
    backup_id = resource.Body('backup_id')
    #: Backup name
    backup_name = resource.Body('backup_name')
    #: ID of the resource to be restored
    target_resource_id = resource.Body('target_resource_id')
    #: Name of the resource to be restored
    target_resource_name = resource.Body('target_resource_name')


class ExtendInfoVaultDelete(resource.Resource):
    #: Properties
    #: Number of resources that fail to be deleted
    fail_count = resource.Body('fail_count')
    #: Number of deleted backups
    total_count = resource.Body('total_count')


class ResourceSpec(resource.Resource):
    #: Properties
    #: Additional information of the resource
    extra_info = resource.Body('extra_info', type=dict)
    #: ID of the resource to be backed up
    id = resource.Body('id')
    #: Name of the resource to be backed up
    name = resource.Body('name')
    #: Type of the resource to be backed up
    type = resource.Body('type')


class ExtendInfoRemoveResources(resource.Resource):
    #: Properties
    #: Number of resources that fail to be deleted
    fail_count = resource.Body('fail_count')
    #: Number of deleted backups
    total_count = resource.Body('total_count')
    #: Resources
    resources = resource.Body('resources', type=list, list_type=ResourceSpec)


class ExtraInfoSpec(resource.Resource):
    #: Properties
    #: Extended parameters of backup
    backup = resource.Body('backup', type=ExtendInfoBackup)
    #: Common parameters
    common = resource.Body('common', type=ExtendInfoCommon)
    #: Extended parameters of deletion
    delete = resource.Body('delete', type=ExtendInfoDelete)
    #: Extended parameters of synchronization
    sync = resource.Body('sync', type=ExtendInfoSync)
    #: Extended parameters of removing resources from a vault
    remove_resources = resource.Body('remove_resources',
                                     type=ExtendInfoRemoveResources)
    #: Resource information
    resource_info = resource.Body('resource', type=ResourceSpec)
    #: Extended parameters of restoration
    restore = resource.Body('restore', type=ExtendInfoRestore)
    #: Extended parameters of deleting a vault
    vault_delete = resource.Body('vault_delete', type=ExtendInfoVaultDelete)


class Task(resource.Resource):
    """CBR Task Resource"""
    resource_key = 'operation_log'
    resources_key = 'operation_logs'
    base_path = '/operation-logs'

    # capabilities
    allow_list = True
    allow_fetch = True

    _query_mapping = resource.QueryParameters('operation_log_id', 'project_id')

    #: Properties
    #: Backup record ID
    checkpoint_id = resource.Body('checkpoint_id')
    #: Creation time
    created_at = resource.Body('created_at')
    #: Task end time
    ended_at = resource.Body('ended_at')
    #: Task error message
    error_info = resource.Body('error_info', type=ErrorInfoSpec)
    #: Task extension information
    extra_info = resource.Body('extra_info', type=ExtraInfoSpec)
    #: Task ID
    id = resource.Body('id')
    #: Task type
    operation_type = resource.Body('operation_type')
    #: Policy ID
    policy_id = resource.Body('policy_id')
    #: Project ID
    project_id = resource.Body('project_id')
    #: Backup provider ID
    provider_id = resource.Body('provider_id')
    #: Task start time
    started_at = resource.Body('started_at')
    #: Task status
    status = resource.Body('status')
    #: Modification time
    updated_at = resource.Body('updated_at')
    #: ID of the vault with which the target resource is associated
    vault_id = resource.Body('vault_id')
    #: Name of the vault with which the target resource is associated
    vault_name = resource.Body('vault_name')



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from otcextensions.tests.functional.sdk.apig import TestApiG


class TestApiGroup(TestApiG):
    api_group = None

    def setUp(self):
        super(TestApiGroup, self).setUp()
        self.create_gateway()

    def test_01_create_api_group(self):
        attrs = {
            "name": "api_group_001",
            "remark": "API group 1"
        }
        created = self.client.create_api_group(gateway=TestApiG.gateway,
                                               **attrs)
        self.assertIsNotNone(created.id)
        TestApiGroup.api_group = created

    def test_02_get_api_group(self):
        found = self.client.get_api_group(
            gateway=TestApiG.gateway,
            api_group=TestApiGroup.api_group.id)
        self.assertEqual(TestApiGroup.api_group.name, found.name)

    def test_03_update_api_group(self):
        new_remark = 'Brand new remark'
        attrs = {
            'name': TestApiGroup.api_group.name,
            'remark': new_remark
        }
        updated = self.client.update_api_group(
            gateway=TestApiG.gateway,
            api_group=TestApiGroup.api_group.id,
            **attrs)
        self.assertEqual(updated.remark, new_remark)

    def test_04_list_api_group(self):
        groups = list(self.client.api_groups(gateway=TestApiG.gateway))
        self.assertGreater(len(groups), 1)

    def test_05_verify_api_group(self):
        attrs = {
            "group_name": "api_group_002"
        }
        self.client.verify_api_group_name(gateway=TestApiG.gateway,
                                          **attrs)

    def test_06_delete_api_group(self):
        self.client.delete_api_group(gateway=TestApiG.gateway,
                                     api_group=TestApiGroup.api_group)
        self.delete_gateway()
